@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,7 +12,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
@@ -22,6 +22,10 @@
 
 #include "mgl2/data_cf.h"
 #include "mgl2/pde.h"
+
+#if MGL_HAVE_ARMA
+#include <armadillo>
+#endif
 //-----------------------------------------------------------------------------
 #include <stdarg.h>
 //-----------------------------------------------------------------------------
@@ -34,6 +38,7 @@ class MGL_EXPORT mglData : public mglDataA
 {
 public:
 using mglDataA::Momentum;
+using mglDataA::Last;
 	long nx;		///< number of points in 1st dimensions ('x' dimension)
 	long ny;		///< number of points in 2nd dimensions ('y' dimension)
 	long nz;		///< number of points in 3d dimensions ('z' dimension)
@@ -144,6 +149,17 @@ using mglDataA::Momentum;
 		va_end(vl);
 	}
 
+#if MGL_HAVE_ARMA
+	inline void Set(const arma::vec &d)
+	{	Create(d.n_elem);	for(long i=0;i<nx;i++)	a[i] = d[i];	}
+	inline void Set(const arma::mat &d)
+	{	Create(d.n_rows,d.n_cols);	for(long i=0;i<nx*ny;i++)	a[i] = d[i];	}
+	inline void Set(const arma::cube &d)
+	{	Create(d.n_rows,d.n_cols,d.n_slices);	for(long i=0;i<nx*ny*nz;i++)	a[i] = d[i];	}
+	inline arma::mat arma_mat(long k=0) const { return arma::mat(a+k*nx*ny,ny,nx);  }
+	inline arma::cube arma_cube() const {return arma::cube(a,nx,ny,nz);}
+#endif
+
 	/// Create or recreate the array with specified size and fill it by zero
 	inline void Create(long mx,long my=1,long mz=1)
 	{	mgl_data_create(this,mx,my,mz);	}
@@ -229,6 +245,34 @@ using mglDataA::Momentum;
 	inline void Put(const mglDataA &dat, long i=-1, long j=-1, long k=-1)
 	{	mgl_data_put_dat(this,&dat,i,j,k);	}
 
+	/// Fills data by integer random numbers of uniform distribution in range [lo,hi]
+	inline void RndInteger(long lo, long hi)
+	{	mgl_data_rnd_integer(this, lo, hi);	}
+	/// Fills data by random numbers of uniform distribution in range [lo,hi]
+	inline void RndUniform(mreal lo, mreal hi)
+	{	mgl_data_rnd_uniform(this, lo, hi);	}
+	/// Fills data by random numbers of Bernoulli distribution
+	inline void RndBernoulli(mreal p=0.5)
+	{	mgl_data_rnd_bernoulli(this, p);	}
+	/// Fills data by random numbers of binomial distribution
+	inline void RndBinomial(long trials, mreal p=0.5)
+	{	mgl_data_rnd_binomial(this, trials, p);	}
+	/// Fills data by random numbers of Gaussian distribution
+	inline void RndGaussian(mreal mu=0.0, mreal sigma=1.0)
+	{	mgl_data_rnd_gaussian(this, mu, sigma);	}
+	/// Fills data by random numbers of exponential distribution
+	inline void RndExponential(mreal lambda)
+	{	mgl_data_rnd_exponential(this, lambda);	}
+	/// Fills data by random numbers of discrete distribution according A
+	inline void RndDiscrete(const mglDataA &A)
+	{	mgl_data_rnd_discrete(this, &A);	}
+	/// Shuffles elements or slices of data array
+	inline void RndShuffle(char dir='a')
+	{	mgl_shuffle(this, dir);	}
+	/// Fills data by fractional brownian motions along x-direction
+	inline void RndBrownian(mreal y1, mreal y2, mreal sigma, mreal alpha)
+	{	mgl_data_brownian(this, y1, y2, sigma, alpha);	}
+	
 	/// Read data from tab-separated text file with auto determining size
 	inline bool Read(const char *fname)
 	{	return mgl_data_read(this,fname); }
@@ -253,6 +297,10 @@ using mglDataA::Momentum;
 	/// Scan textual file for template and fill data array
 	inline int ScanFile(const char *fname, const char *templ)
 	{	return mgl_data_scan_file(this,fname, templ);	}
+	/// Read data from binary file of type: 0 - double, 1 - float, 2 - long double, 3 - long int, 4 - int, 5 - short int, 6 - char.
+	/** NOTE: this function may not correctly read binary files written in different CPU kind! */
+	inline bool ReadBin(const char *fname, int type)
+	{	return mgl_data_read_bin(this, fname, type);	}
 
 
 	/// Get column (or slice) of the data filled by formulas of named columns
@@ -292,9 +340,22 @@ using mglDataA::Momentum;
 	/// Create n-th points distribution of this data values in range [v1, v2] with weight w
 	inline mglData Hist(const mglDataA &w, long n,mreal v1=0,mreal v2=1, long nsub=0) const
 	{	return mglData(true,mgl_data_hist_w(this,&w,n,v1,v2,nsub));	}
+	/// Create n-th points distribution of this data values in range [v1, v2] with weight w by using linear interpolation between coordinate.
+	inline mglData HistL(long n,mreal v1=0,mreal v2=1) const
+	{	return mglData(true,mgl_data_hist_l(this,NULL,n,v1,v2));	}
+	/// Create n-th points distribution of this data values in range [v1, v2] by using linear interpolation between coordinate.
+	inline mglData HistL(const mglDataA &w, long n,mreal v1=0,mreal v2=1) const
+	{	return mglData(true,mgl_data_hist_l(this,&w,n,v1,v2));	}
 	/// Get array which is result of summation in given direction or directions
 	inline mglData Sum(const char *dir) const
 	{	return mglData(true,mgl_data_sum(this,dir));	}
+	/// Get array of positions of first value large val
+	inline mglData First(const char *dir, mreal val) const
+	{	return mglData(true,mgl_data_first_dir(this,dir,val));	}
+	/// Get array of positions of last value large val
+	inline mglData Last(const char *dir, mreal val) const
+	{	return mglData(true,mgl_data_last_dir(this,dir,val));	}
+
 	/// Get array which is result of maximal values in given direction or directions
 	inline mglData Max(const char *dir) const
 	{	return mglData(true,mgl_data_max_dir(this,dir));	}
@@ -397,6 +458,29 @@ using mglDataA::Momentum;
 	/// Project the periodical data to range [v1,v2] (like mod() function). Separate branches by NAN if sep=true.
 	inline void Coil(mreal v1, mreal v2, bool sep=true)
 	{	mgl_data_coil(this, v1, v2, sep);	}
+	/// Keep the data sign/value along line i and j in given direction. 
+	/** Parameter "how" may contain: 'x','y' or 'z' for direction (default is 'y'); 'a' for keeping amplitude instead of sign.*/
+	inline void Keep(const char *how, long i, long j=0)
+	{	mgl_data_keep(this, how, i, j);	}
+
+	/// Replace initial mode by found one for equation u_xx+pot*u=0 in waveguide with boundary defined by \a mask.
+	/** Parameter \a how may contain (default is "xyz0"):
+	* ‘x‘ for 1d case with ny*nz variants,
+	* ‘y‘ for 2d case with nz variants,
+	* ‘z‘ for 3d case, TODO
+	* ‘0‘ for zero boundary: u[n] = 0,
+	* ‘1‘ for free boundary du/dx=0: u[n] = u[n-1],
+	* ‘2‘ for zero laplace boundary d^2u/dx^2=0: u[n] = 2*u[n-1]-u[n-2],
+	* ‘s‘ for 2nd order boundary du/dx=k: u[n] = (4*u[n-1]-u[n-2]-2*kdx)/3,
+	* ‘r‘ for radiation boundary du/dx=ku: u[n]=(4*u[n-1]-u[n-2])/(3+2*kdx),
+	* ‘c‘ for cyclic boundary ( u[n] = u[0] ignore \a kdx),
+	* '+' for focusing nonlinearity,
+	* '-' for defocusing nonlinearity.
+	* Parameter \a iter set maximal number of iterations (0 is automatic). */
+	inline void Mode(const mglDataA &mask, const mglDataA &pot, const char *how, double kdx=0, long iter=0)
+	{	mgl_data_mode(this, &mask, &pot, how, kdx, iter);	}
+	inline void Mode(const mglDataA &mask, const char *how, double kdx=0, long iter=0)
+	{	mgl_data_mode(this, &mask, NULL, how, kdx, iter);	}
 
 	/// Apply Hankel transform
 	inline void Hankel(const char *dir)	{	mgl_data_hankel(this,dir);	}
@@ -561,9 +645,12 @@ inline mglData mglQO3d(const char *ham, const mglDataA &ini_re, const mglDataA &
 /// Finds ray with starting point r0, p0 (and prepares ray data for mglQO2d)
 inline mglData mglRay(const char *ham, mglPoint r0, mglPoint p0, mreal dt=0.1, mreal tmax=10)
 {	return mglData(true, mgl_ray_trace(ham, r0.x, r0.y, r0.z, p0.x, p0.y, p0.z, dt, tmax));	}
-/// Saves result of ODE solving for var complex variables with right part func (separated by ';') and initial conditions x0 over time interval [0,tmax] with time step dt
-inline mglData mglODE(const char *func, const char *var, const mglDataA &ini, mreal dt=0.1, mreal tmax=10)
-{	return mglData(true, mgl_ode_solve_str(func,var, &ini, dt, tmax));	}
+/// Saves result of ODE solving for var complex variables with right part func (separated by ';') and initial conditions x0 over time interval [0,tmax] with time step dt. Functions jump are called each steps to handle special conditions (like reflection at boundary).
+inline mglData mglODE(const char *func, const char *var, const mglDataA &ini, mreal dt=0.1, mreal tmax=10, const char *jump=NULL)
+{	return mglData(true, mgl_ode_solve_str_b(func,var, &ini, dt, tmax, jump));	}
+/// Saves result of ODE solving for var complex variables with right part func (separated by ';') and initial conditions x0 of size n*m over time interval [0,tmax] with time step dt
+inline mglData mglODEs(const char *func, const char *var, char brd, const mglDataA &ini, mreal dt=0.1, mreal tmax=10)
+{	return mglData(true, mgl_ode_solve_set(func,var, brd, &ini, dt, tmax));	}
 //-----------------------------------------------------------------------------
 /// Get array as solution of tridiagonal system of equations a[i]*x[i-1]+b[i]*x[i]+c[i]*x[i+1]=d[i]
 /** String \a how may contain:
@@ -624,12 +711,17 @@ inline mglData mglGSplineInit(const mglDataA &xdat, const mglDataA &ydat)
 inline mreal mglGSpline(const mglDataA &coef, mreal dx, mreal *d1=0, mreal *d2=0)
 {	return mgl_gspline(&coef, dx, d1,d2);	}
 //-----------------------------------------------------------------------------
+/// Evaluate formula 'str' for given list of variables 'vars'. 
+/** NOTE: you need to delete returned data array!*/
+HMDT MGL_EXPORT mglFormulaCalc(const char *str, const std::vector<mglDataA*> &vars);
+//-----------------------------------------------------------------------------
+extern long mgl_omp_thr;	// threshold for use OpenMP in CalcV(). NOTE: default value is 10000 for 8...12 cores CPU
 /// Wrapper class for expression evaluating
 class MGL_EXPORT mglExpr
 {
 	HMEX ex;
-	mglExpr(const mglExpr &){}	// copying is not allowed
-	const mglExpr &operator=(const mglExpr &t){return t;}	// copying is not allowed
+	mglExpr(const mglExpr &){ex=0;}	// copying is not allowed
+	const mglExpr &operator=(const mglExpr &t){ex=0;	return t;}	// copying is not allowed
 public:
 	mglExpr(const char *expr)		{	ex = mgl_create_expr(expr);	}
 #if MGL_HAVE_RVAL
@@ -642,14 +734,16 @@ public:
 	/// Return value of expression differentiation over variable dir for given x,y,z variables
 	inline double Diff(char dir, double x, double y=0, double z=0)
 	{	return mgl_expr_diff(ex,dir, x,y,z);	}
-#ifndef SWIG
 	/// Return value of expression for given variables
-	inline double Eval(mreal var[26])
+	inline double Eval(mreal var[MGL_VS])
 	{	return mgl_expr_eval_v(ex,var);	}
 	/// Return value of expression differentiation over variable dir for given variables
-	inline double Diff(char dir, mreal var[26])
+	inline double Diff(char dir, mreal var[MGL_VS])
 	{	return mgl_expr_diff_v(ex,dir, var);	}
-#endif
+	/// Return value of expression for given variables
+	inline void CalcV(HMDT res, HCDT vars[MGL_VS])
+	{	if(res->GetNN()<mgl_omp_thr)	mgl_expr_eval_dat(ex, res, vars);
+		else	mgl_expr_eval_omp(ex, res, vars);	}
 };
 //-----------------------------------------------------------------------------
 /// Class which present equidistantly distributed data
@@ -816,7 +910,7 @@ public:
 
 	/// Create or recreate the array with specified size and fill it by zero
 	inline void Create(long mx,long my=1,long mz=1)	{	nx=mx;	ny=my;	nz=mz;	setD();	}
-	inline void SetRanges(mglPoint p1, mglPoint p2)	{	v1=p1;	v2=p2;	setD();	}
+	inline void SetRanges(const mglPoint &p1, const mglPoint &p2)	{	v1=p1;	v2=p2;	setD();	}
 	/// Set formula to be used as dfunction
 	inline void SetFormula(const char *eq)
 	{
@@ -907,7 +1001,7 @@ class MGL_EXPORT mglDataT : public mglDataA
 {
 	const mglDataA &dat;
 	long ind;
-	const mglDataT &operator=(const mglDataT &d)	{	return d;	}
+	const mglDataT &operator=(const mglDataT &d)	{	ind=0;	return d;	}
 public:
 	mglDataT(const mglDataT &d) : dat(d.dat), ind(d.ind)	{	s = d.s;	}
 	mglDataT(const mglDataA &d, long col=0) : dat(d), ind(col)	{}
@@ -957,7 +1051,7 @@ class MGL_EXPORT mglDataR : public mglDataA
 {
 	const mglDataA &dat;
 	long ind;
-	const mglDataR &operator=(const mglDataR &d)	{	return d;	}
+	const mglDataR &operator=(const mglDataR &d)	{	ind=0;	return d;	}
 public:
 	mglDataR(const mglDataR &d) : dat(d.dat), ind(d.ind)	{	s = d.s;	}
 	mglDataR(const mglDataA &d, long row=0) : dat(d), ind(row)	{}
