@@ -348,6 +348,12 @@ using mglDataA::Last;
 	/// Create n-th points distribution of this data values in range [v1, v2] with weight w
 	inline mglData Hist(const mglDataA &w, long n,mreal v1=0,mreal v2=1, long nsub=0) const
 	{	return mglData(true,mgl_data_hist_w(this,&w,n,v1,v2,nsub));	}
+	/// Create n-th points distribution of this data values in range [v1, v2] with weight w by using linear interpolation between coordinate.
+	inline mglData HistL(long n,mreal v1=0,mreal v2=1) const
+	{	return mglData(true,mgl_data_hist_l(this,NULL,n,v1,v2));	}
+	/// Create n-th points distribution of this data values in range [v1, v2] by using linear interpolation between coordinate.
+	inline mglData HistL(const mglDataA &w, long n,mreal v1=0,mreal v2=1) const
+	{	return mglData(true,mgl_data_hist_l(this,&w,n,v1,v2));	}
 	/// Get array of positions of first value, which abs() is  large val
 	inline mglData First(const char *dir, mreal val) const
 	{	return mglData(true,mgl_data_first_dir(this,dir,val));	}
@@ -406,6 +412,26 @@ using mglDataA::Last;
 	inline void Keep(const char *how, long i, long j=0)
 	{	mgl_datac_keep(this, how, i, j);	}
 
+	/// Replace initial mode by found one for equation u_xx+pot*u=0 in waveguide with boundary defined by \a mask.
+	/** Parameter \a how may contain (default is "xyz0"):
+	* ‘x‘ for 1d case with ny*nz variants,
+	* ‘y‘ for 2d case with nz variants,
+	* ‘z‘ for 3d case, TODO
+	* ‘0‘ for zero boundary: u[n] = 0,
+	* ‘1‘ for free boundary du/dx=0: u[n] = u[n-1],
+	* ‘2‘ for zero laplace boundary d^2u/dx^2=0: u[n] = 2*u[n-1]-u[n-2],
+	* ‘s‘ for 2nd order boundary du/dx=k: u[n] = (4*u[n-1]-u[n-2]-2*kdx)/3,
+	* ‘r‘ for radiation boundary du/dx=ku: u[n]=(4*u[n-1]-u[n-2])/(3+2*kdx),
+	* ‘i‘ for radiation boundary du/dx=iku: u[n]=(4*u[n-1]-u[n-2])/(3+2i*kdx),
+	* ‘c‘ for cyclic boundary ( u[n] = u[0] ignore \a kdx),
+	* '+' for focusing nonlinearity,
+	* '-' for defocusing nonlinearity.
+	* Parameter \a iter set maximal number of iterations (0 is automatic). */
+	inline void Mode(const mglDataA &mask, const mglDataA &pot, const char *how, dual kdx=0., long iter=0)
+	{	mgl_datac_mode(this, &mask, &pot, how, kdx, iter);	}
+	inline void Mode(const mglDataA &mask, const char *how, dual kdx=0., long iter=0)
+	{	mgl_datac_mode(this, &mask, NULL, how, kdx, iter);	}
+
 	/// Set as the data envelop
 	inline void Envelop(char dir='x')	{	mgl_datac_envelop(this,dir);	}
 	/// Hankel transform
@@ -474,11 +500,11 @@ using mglDataA::Last;
 	{	if(this!=&d)	Set(&d);	return d;	}
 	inline dual operator=(dual val)
 	{
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	a[i]=val;	return val;	}
 	inline dual operator=(mreal val)
 	{
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	a[i]=val;	return val;	}
 	/// Multiply the data by other one for each element
 	inline void operator*=(const mglDataA &d)	{	mgl_datac_mul_dat(this,&d);	}
@@ -557,9 +583,9 @@ inline mglDataC mglQO3dc(const char *ham, const mglDataA &ini_re, const mglDataA
 {	return mglDataC(true, mgl_qo3d_solve_c(ham, &ini_re, &ini_im, &ray, r, k0, 0, 0, 0));	}
 inline mglDataC mglQO3dc(const char *ham, const mglDataA &ini_re, const mglDataA &ini_im, const mglDataA &ray, mglData &xx, mglData &yy, mglData &zz, mreal r=1, mreal k0=100)
 {	return mglDataC(true, mgl_qo3d_solve_c(ham, &ini_re, &ini_im, &ray, r, k0, &xx, &yy, &zz));	}
-/// Saves result of ODE solving for var complex variables with right part func (separated by ';') and initial conditions x0 over time interval [0,tmax] with time step dt
-inline mglDataC mglODEc(const char *func, const char *var, const mglDataA &ini, mreal dt=0.1, mreal tmax=10)
-{	return mglDataC(true, mgl_ode_solve_str_c(func,var, &ini, dt, tmax));	}
+/// Saves result of ODE solving for var complex variables with right part func (separated by ';') and initial conditions x0 over time interval [0,tmax] with time step dt. Functions jump are called each steps to handle special conditions (like reflection at boundary).
+inline mglDataC mglODEc(const char *func, const char *var, const mglDataA &ini, mreal dt=0.1, mreal tmax=10, const char *jump=NULL)
+{	return mglDataC(true, mgl_ode_solve_str_cb(func,var, &ini, dt, tmax, jump));	}
 /// Saves result of ODE solving for var complex variables with right part func (separated by ';') and initial conditions x0 of size n*m over time interval [0,tmax] with time step dt
 inline mglDataC mglODEcs(const char *func, const char *var, char brd, const mglDataA &ini, mreal dt=0.1, mreal tmax=10)
 {	return mglDataC(true, mgl_ode_solve_set_c(func,var, brd, &ini, dt, tmax));	}
@@ -598,6 +624,32 @@ HADT MGL_EXPORT mglFormulaCalcC(const char *str, const std::vector<mglDataA*> &v
 #define _DC_		((mglDataC *)*d)
 //-----------------------------------------------------------------------------
 #ifndef SWIG
+inline mglDataC operator*(const mglDataC &b, const mglDataA &d)
+{	mglDataC a(&b);	a*=d;	return a;	}
+inline mglDataC operator*(dual b, const mglDataC &d)
+{	mglDataC a(&d);	a*=b;	return a;	}
+inline mglDataC operator*(const mglDataC &d, dual b)
+{	mglDataC a(&d);	a*=b;	return a;	}
+inline mglDataC operator-(const mglDataC &b, const mglDataA &d)
+{	mglDataC a(&b);	a-=d;	return a;	}
+inline mglDataC operator-(dual b, const mglDataC &d)
+{	mglDataC a(&d);	a-=b;	return a;	}
+inline mglDataC operator-(const mglDataC &d, dual b)
+{	mglDataC a(&d);	a-=b;	return a;	}
+inline mglDataC operator+(const mglDataC &b, const mglDataA &d)
+{	mglDataC a(&b);	a+=d;	return a;	}
+inline mglDataC operator+(dual b, const mglDataC &d)
+{	mglDataC a(&d);	a+=b;	return a;	}
+inline mglDataC operator+(const mglDataC &d, dual b)
+{	mglDataC a(&d);	a+=b;	return a;	}
+inline mglDataC operator/(const mglDataA &b, const mglDataC &d)
+{	mglDataC a(&b);	a/=d;	return a;	}
+inline mglDataC operator/(const mglDataC &d, dual b)
+{	mglDataC a(&d);	a/=b;	return a;	}
+inline bool operator==(const mglDataC &b, const mglDataC &d)
+{	if(b.nx!=d.nx || b.ny!=d.ny || b.nz!=d.nz)	return false;
+	return !memcmp(b.a,d.a,b.nx*b.ny*b.nz*sizeof(dual));	}
+//-----------------------------------------------------------------------------
 /// Wrapper class for complex expression evaluating
 class MGL_EXPORT mglExprC
 {
@@ -613,13 +665,17 @@ public:
 	/// Return value of expression for given x,y,z,u,v,w variables
 	inline dual Eval(dual x, dual y, dual z, dual u, dual v, dual w)
 	{
-		mdual var[26];
+		mdual var[MGL_VS];
 		var['x'-'a']=x;	var['y'-'a']=y;	var['z'-'a']=z;
 		var['u'-'a']=u;	var['v'-'a']=v;	var['w'-'a']=w;
 		return mgl_cexpr_eval_v(ex,var);	}
 	/// Return value of expression for given variables
-	inline dual Eval(dual var[26])
+	inline dual Eval(dual var[MGL_VS])
 	{	return mgl_cexpr_eval_v(ex,reinterpret_cast<mdual*>(var));	}
+	/// Return value of expression for given variables
+	inline void CalcV(HADT res, HCDT vars[MGL_VS])
+	{	if(res->GetNN()<mgl_omp_thr)	mgl_cexpr_eval_dat(ex, res, vars);
+		else	mgl_cexpr_eval_omp(ex, res, vars);	}
 };
 #endif
 //-----------------------------------------------------------------------------
