@@ -793,7 +793,8 @@ public:
 		static std::string str;
 		const char *c = cmd->text();
 		desc->label(Parse->CmdDesc(c));
-		str = helpname+c;
+		str = helpname+"#"+c;
+//		str = helpname+"#index-"+c;
 		help->load(str.c_str());
 		std::string par = Parse->CmdFormat(c), cname;
 		std::vector<std::string> vars;
