@@ -17,7 +17,6 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#include <ctype.h>
 #include "mgl2/datac.h"
 #include "mgl2/evalc.h"
 #include "mgl2/thread.h"
@@ -32,23 +31,23 @@ HADT MGL_EXPORT mgl_datac_trace(HCDT d)
 	if(dc)
 	{
 		if(ny>=nx && nz>=nx)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i=0;i<nx;i++)	r->a[i] = dc->a[i+nx*(i+ny*i)];
 		else if(ny>=nx)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i=0;i<nx;i++)	r->a[i] = dc->a[i+nx*i];
 		else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i=0;i<nx;i++)	r->a[i] = dc->a[i];
 	}
 	else if(ny>=nx && nz>=nx)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)	r->a[i] = d->v(i,i,i);
 	else if(ny>=nx)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)	r->a[i] = d->v(i,i);
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)	r->a[i] = d->v(i);
 	return r;
 }
@@ -119,14 +118,14 @@ HADT MGL_EXPORT mgl_datac_subdata_ext(HCDT d, HCDT xx, HCDT yy, HCDT zz)
 		if(!iz)	{	zz = &tz;	if(vz>=0)	tz.Fill(vz);	else tz.All();	}
 		r=new mglDataC(n,m,l);
 		if(dd)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i0=0;i0<n*m*l;i0++)
 			{
 				long x=long(floor(0.5+xx->vthr(i0))), y=long(floor(0.5+yy->vthr(i0))), z=long(floor(0.5+zz->vthr(i0)));
 				r->a[i0] = (x>=0 && x<nx && y>=0 && y<ny && z>=0 && z<nz)?dd->a[x+nx*(y+ny*z)]:NAN;
 			}
 		else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i0=0;i0<n*m*l;i0++)
 			{
 				long x=long(floor(0.5+xx->vthr(i0))), y=long(floor(0.5+yy->vthr(i0))), z=long(floor(0.5+zz->vthr(i0)));
@@ -141,14 +140,14 @@ HADT MGL_EXPORT mgl_datac_subdata_ext(HCDT d, HCDT xx, HCDT yy, HCDT zz)
 		if(zz->GetNx()>1 || vz>=0)	l=zz->GetNx();	else	{	l=nz;	zz = &tz;	}
 		r=new mglDataC(n,m,l);
 		if(dd)
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<l;k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 			{
 				long x=long(floor(0.5+xx->v(i))), y=long(floor(0.5+yy->v(j))), z=long(floor(0.5+zz->v(k)));
 				r->a[i+n*(j+m*k)] = (x>=0 && x<nx && y>=0 && y<ny && z>=0 && z<nz)?dd->a[x+nx*(y+ny*z)]:NAN;
 			}
 		else
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<l;k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 			{
 				long x=long(floor(0.5+xx->v(i))), y=long(floor(0.5+yy->v(j))), z=long(floor(0.5+zz->v(k)));
@@ -170,14 +169,14 @@ HADT MGL_EXPORT mgl_datac_subdata(HCDT d, long xx,long yy,long zz)
 	const mglDataC *dd = dynamic_cast<const mglDataC *>(d);
 	mglDataC *r=new mglDataC(n,m,l);
 	if(xx>=nx || yy>=ny || zz>=nz)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<n*m*l;i++)	r->a[i] = NAN;
 	else if(dd)
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<l;k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 			r->a[i+n*(j+m*k)] = dd->a[xx+dx*i + nx*(yy+dy*j + ny*(zz+dz*k))];
 	else
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<l;k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 			r->a[i+n*(j+m*k)] = d->v(xx+dx*i, yy+dy*j, zz+dz*k);
 	if(m==1)	{	r->ny=r->nz;	r->nz=1;	}// "squeeze" dimensions
@@ -199,7 +198,7 @@ static void *mgl_cresize(void *par)
 	const dual *a=t->b;
 	const mreal *c=(const mreal *)t->v;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
@@ -244,19 +243,19 @@ HADT MGL_EXPORT mgl_datac_combine(HCDT d1, HCDT d2)
 	const mglDataC *c1=dynamic_cast<const mglDataC *>(d1);
 	const mglDataC *c2=dynamic_cast<const mglDataC *>(d2);
 	if(c1 && c2)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long j=0;j<n2;j++)	for(long i=0;i<n1;i++)
 			r->a[i+n1*j] = c1->a[i]*c2->a[j];
 	else if(c1)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long j=0;j<n2;j++)	for(long i=0;i<n1;i++)
 			r->a[i+n1*j] = c1->a[i]*d2->vthr(j);
 	else if(c2)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long j=0;j<n2;j++)	for(long i=0;i<n1;i++)
 			r->a[i+n1*j] = d1->vthr(i)*c2->a[j];
 	else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long j=0;j<n2;j++)	for(long i=0;i<n1;i++)
 			r->a[i+n1*j] = d1->vthr(i)*d2->vthr(j);
 	return r;
@@ -271,7 +270,7 @@ static void *mgl_sumc_z(void *par)
 	dual *b=t->a;
 	const dual *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -288,7 +287,7 @@ static void *mgl_sumc_y(void *par)
 	dual *b=t->a;
 	const dual *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -305,7 +304,7 @@ static void *mgl_sumc_x(void *par)
 	dual *b=t->a;
 	const dual *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -326,7 +325,7 @@ HADT MGL_EXPORT mgl_datac_sum(HCDT dat, const char *dir)
 	const mglDataC *d=dynamic_cast<const mglDataC *>(dat);
 	if(d)	memcpy(c,d->a,nx*ny*nz*sizeof(dual));
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	c[i]=dat->vthr(i);
 
 	if(strchr(dir,'z') && nz>1)
@@ -368,7 +367,7 @@ HADT MGL_EXPORT mgl_datac_momentum(HCDT dat, char dir, const char *how)
 	if(dir=='x')
 	{
 		b=new mglDataC(nx);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)
 		{
 			dual i1=0,i0=0;
@@ -383,7 +382,7 @@ HADT MGL_EXPORT mgl_datac_momentum(HCDT dat, char dir, const char *how)
 	if(dir=='y')
 	{
 		b=new mglDataC(ny);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<ny;i++)
 		{
 			dual i1=0,i0=0;
@@ -399,7 +398,7 @@ HADT MGL_EXPORT mgl_datac_momentum(HCDT dat, char dir, const char *how)
 	{
 		long nn=nx*ny;
 		b=new mglDataC(nz);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nz;i++)
 		{
 			dual i1=0,i0=0;
@@ -427,21 +426,21 @@ HADT MGL_EXPORT mgl_datac_evaluate(HCDT dat, HCDT idat, HCDT jdat, HCDT kdat, in
 	mreal dx = nx-1, dy = ny-1, dz = nz-1;
 	if(!norm)	dx=dy=dz=1;
 	if(dd)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<idat->GetNN();i++)
 		{
 			mreal x=dx*idat->vthr(i), y=jdat?dy*jdat->vthr(i):0, z=kdat?dz*kdat->vthr(i):0;
 			r->a[i] = mgl_isnum(x*y*z)?mglSpline3st<mreal>(dd->a,nx,ny,nz, x,y,z):NAN;
 		}
 	else if(dc)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<idat->GetNN();i++)
 		{
 			mreal x=dx*idat->vthr(i), y=jdat?dy*jdat->vthr(i):0, z=kdat?dz*kdat->vthr(i):0;
 			r->a[i] = mgl_isnum(x*y*z)?mglSpline3st<dual>(dc->a,nx,ny,nz, x,y,z):NAN;
 		}
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<idat->GetNN();i++)
 		{
 			mreal x=dx*idat->vthr(i), y=jdat?dy*jdat->vthr(i):0, z=kdat?dz*kdat->vthr(i):0;
@@ -484,7 +483,7 @@ void MGL_EXPORT mgl_datac_mul_dat(HADT d, HCDT a)
 	if(mz*my*mx==1)
 	{
 		dual v=c?c->a[0]:a->v(0);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	d->a[i] += v;
 	}
 	else
@@ -494,10 +493,10 @@ void MGL_EXPORT mgl_datac_mul_dat(HADT d, HCDT a)
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
 		if(c)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] *= c->a[i];
 		else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] *= a->vthr(i);
 	}
 }
@@ -518,7 +517,7 @@ void MGL_EXPORT mgl_datac_div_dat(HADT d, HCDT a)
 	if(mz*my*mx==1)
 	{
 		dual v=c?c->a[0]:a->v(0);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	d->a[i] /= v;
 	}
 	else
@@ -528,10 +527,10 @@ void MGL_EXPORT mgl_datac_div_dat(HADT d, HCDT a)
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
 		if(c)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] /= c->a[i];
 		else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] /= a->vthr(i);
 	}
 }
@@ -552,7 +551,7 @@ void MGL_EXPORT mgl_datac_add_dat(HADT d, HCDT a)
 	if(mz*my*mx==1)
 	{
 		dual v=c?c->a[0]:a->v(0);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	d->a[i] += v;
 	}
 	else
@@ -562,10 +561,10 @@ void MGL_EXPORT mgl_datac_add_dat(HADT d, HCDT a)
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
 		if(c)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] += c->a[i];
 		else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] += a->vthr(i);
 	}
 }
@@ -586,7 +585,7 @@ void MGL_EXPORT mgl_datac_sub_dat(HADT d, HCDT a)
 	if(mz*my*mx==1)
 	{
 		dual v=c?c->a[0]:a->v(0);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	d->a[i] -= v;
 	}
 	else
@@ -596,10 +595,10 @@ void MGL_EXPORT mgl_datac_sub_dat(HADT d, HCDT a)
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
 		if(c)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] -= c->a[i];
 		else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] -= a->vthr(i);
 	}
 }
@@ -607,7 +606,7 @@ void MGL_EXPORT mgl_datac_sub_dat(HADT d, HCDT a)
 void MGL_EXPORT mgl_datac_sub_num(HADT d, mdual a)
 {
 	long n=d->GetNN();
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=0;i<n;i++)	d->a[i] -= dual(a);
 }
 //-----------------------------------------------------------------------------
@@ -657,7 +656,7 @@ HADT MGL_EXPORT mgl_datac_section(HCDT dat, HCDT ids, char dir, mreal val)
 		for(size_t s=0;s<ns;s++)
 		{
 			long pp = ps[s];
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<nz;k++)	for(long j=0;j<ls[s];j++)	for(long i=0;i<nx;i++)
 				r->a[i+nx*(sh+j+nl*k)] = dat->vc(i,pp+j,k);
 			sh += ls[s];
@@ -670,7 +669,7 @@ HADT MGL_EXPORT mgl_datac_section(HCDT dat, HCDT ids, char dir, mreal val)
 		for(size_t s=0;s<ns;s++)
 		{
 			long pp = ps[s];
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<ls[s];i++)
 				r->a[sh+i+nl*(j+ny*k)] = dat->vc(pp+i,j,k);
 			sh += ls[s];
@@ -683,7 +682,7 @@ HADT MGL_EXPORT mgl_datac_section(HCDT dat, HCDT ids, char dir, mreal val)
 		for(size_t s=0;s<ns;s++)
 		{
 			long pp = ps[s];
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<ls[s];k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 				r->a[i+nx*(j+ny*(sh+k))] = dat->vc(i,j,pp+k);
 			sh += ls[s];
@@ -773,3 +772,271 @@ void MGL_EXPORT mgl_datac_keep_(uintptr_t *d, const char *how, long *i, long *j,
 {	char *s=new char[l+1];	memcpy(s,how,l);	s[l]=0;
 	mgl_datac_keep(_DC_, s, *i, *j);	delete []s;	}
 //-----------------------------------------------------------------------------
+struct bndI
+{	long i0, i1, i2, i3;
+	bndI(long j0,long j1,long j2=0,long j3=0):i0(j0),i1(j1),i2(j2),i3(j3)	{}
+};
+//-----------------------------------------------------------------------------
+void mgl_collect_mask1d(std::vector<bndI> &bnd, HCDT mask, long j)
+{
+	long nx = mask->GetNx();
+	double v1,v2=mask->v(0,j);
+	bnd.clear();
+	for(long i=1;i<nx-1;i++)
+	{
+		v1=v2;	v2=mask->v(i,j);
+		if(v1 && !v2)	bnd.push_back(bndI(i,i-1,i-2));
+		if(!v1 && v2)	bnd.push_back(bndI(i-1,i,i+1));
+	}
+	if(bnd.empty())
+	{	bnd.push_back(bndI(0,1,2));	bnd.push_back(bndI(nx-1,nx-2,nx-3));	}
+}
+void mgl_mode1c(HADT d, HCDT mask, HCDT pot, const long nx, const long nn, const int kind, const dual kdx, const long flag, long iter)
+{
+	if(mask->GetNx()!=nx || pot->GetNx()!=nx)	return;
+	if(iter<1)	iter = nx*nx/(1-iter);
+	bool both = mask->GetNN() == nx*nn;
+	bool pth = pot->GetNN() == nx*nn;
+	bool ncycl = !(flag&1);
+	double nlin=0;
+	if(flag&2)	nlin = 1;
+	if(flag&4)	nlin =-1;
+	const double k2 = norm(kdx);
+	dual c1=0, c2=0, cs=0;
+	switch(kind)
+	{
+		case 1:	c1=1.;	break;
+		case 2:	c1=2.;	c2 = -1.;	break;
+		case 3:	c1=4./3.;	c2 = -1/3.;	cs = (-2./3.)*kdx;	break;
+		case 4:	c1=4./(3.-2.*kdx);		c2=-1./(3.-2.*kdx);	break;
+		case 5:	c1=4./(3.-dual(0,2)*kdx);	c2=-1./(3.-dual(0,2)*kdx);	break;
+	}
+
+	std::vector<bndI> bnd;
+	if(ncycl && !both)	// the same mask for all slices
+		mgl_collect_mask1d(bnd, mask,0);
+	dual *res = new dual[nx];
+	for(long j=0;j<nn;j++)
+	{
+		dual *aa = d->a + j*nx;
+		long i0 = j*nx;
+		if(ncycl && both)	// different mask for each slice
+			mgl_collect_mask1d(bnd, mask,j);
+		double p0 = 0;
+		for(long i=0;i<nx;i++)	p0 += norm(aa[i]);
+		if(p0==0)	continue;	// nothing to do since inifield is zero
+		for(long kk=0;kk<iter;kk++)
+		{
+#pragma omp parallel for schedule(static)
+			for(long i=1;i<nx-1;i++)
+			{
+				dual v = aa[i];
+				res[i] = (aa[i-1] + v + aa[i+1])/3. + k2*( nlin*norm(v) + pot->vthr(pth?i+i0:i) )*v;
+				res[i] *= mask->vthr(both?i+i0:i);
+			}
+			if(ncycl)	for(size_t i=0;i<bnd.size();i++)
+			{
+				const bndI &bb = bnd[i];
+				res[bb.i0] = c1*res[bb.i1] + c2*res[bb.i2] + cs;
+			}
+			else
+			{
+				dual v = aa[0];
+				res[0] = (aa[nx-1] + v + aa[1])/3. + k2*( nlin*norm(v) + pot->vthr(pth?i0:0) )*v;
+				res[0] *= mask->vthr(both?i0:0);
+				const long i = nx-1;	v = aa[i];
+				res[i] = (aa[i-1] + v + aa[0])/3. + k2*( nlin*norm(v) + pot->vthr(pth?i+i0:i) )*v;
+				res[i] *= mask->vthr(both?i+i0:i);
+			}
+			double p=0;
+			for(long i=0;i<nx;i++)	p += norm(res[i]);
+			p = sqrt(p0/p);
+			for(long i=0;i<nx;i++)	aa[i] = res[i]*p;
+		}
+	}
+	delete []res;
+}
+//-----------------------------------------------------------------------------
+void mgl_collect_mask2d(std::vector<bndI> &h, std::vector<bndI> &d, std::vector<bndI> &s, std::vector<bndI> &e, HCDT mask, long k)
+{
+	long nx = mask->GetNx(), ny = mask->GetNy();
+	h.clear();	d.clear();	s.clear();	e.clear();
+	for(long j=1;j<ny-1;j++)	for(long i=1;i<nx-1;i++)
+	{
+		if(mask->v(i,j,k))	continue;	// inside
+		int vu=mask->v(i,j-1,k), vl=mask->v(i-1,j,k), vr=mask->v(i+1,j,k), vb=mask->v(i,j+1,k);
+		int sum = vu+vl+vr+vb;
+		long i0 = i+nx*(j+ny*k);
+		switch(sum)
+		{
+		case 1:
+			if(vu)	h.push_back(bndI(i0,i0-nx,i0-nx*2));
+			if(vl)	h.push_back(bndI(i0,i0-1,i0-2));
+			if(vr)	h.push_back(bndI(i0,i0+1,i0+2));
+			if(vb)	h.push_back(bndI(i0,i0+nx,i0-nx*2));
+			break;
+		case 2:
+			if(vu && vl)	d.push_back(bndI(i0,i0-nx,i0-1,i0-nx-1));
+			if(vu && vr)	d.push_back(bndI(i0,i0-nx,i0+1,i0-nx+1));
+			if(vb && vl)	d.push_back(bndI(i0,i0+nx,i0-1,i0+nx-1));
+			if(vb && vr)	d.push_back(bndI(i0,i0+nx,i0+1,i0+nx+1));
+			if(vu && vb)	s.push_back(bndI(i0,i0-nx,i0+nx));
+			if(vl && vr)	s.push_back(bndI(i0,i0-1,i0+1));
+			break;
+		case 3:
+			if(!vu)	h.push_back(bndI(i0,i0+nx,i0+nx*2));
+			if(!vl)	h.push_back(bndI(i0,i0+1,i0+2));
+			if(!vr)	h.push_back(bndI(i0,i0-1,i0-2));
+			if(!vb)	h.push_back(bndI(i0,i0-nx,i0-nx*2));
+			break;
+		case 4:	e.push_back(bndI(i0,-1));	break;	// remove "dots"
+		}
+	}
+}
+void mgl_mode2c(HADT d, HCDT mask, HCDT pot, const long nx, const long ny, const long nz, const int kind, const dual kdx, const long flag, long iter)
+{
+	if(mask->GetNx()!=nx || pot->GetNx()!=nx || mask->GetNy()!=ny || pot->GetNy()!=ny)	return;
+	if(iter<1)	iter = (nx>=ny?nx*nx:ny*ny)/(1-iter);
+	bool both = mask->GetNN() == nx*ny*nz;
+	bool pth = pot->GetNN() == nx*ny*nz;
+	bool ncycl = !(flag&1);
+	double nlin=0;
+	const dual kdx2 = kdx/M_SQRT2;
+	if(flag&2)	nlin = 1;
+	if(flag&4)	nlin =-1;
+	const double k2 = norm(kdx);
+	dual c1=0, c2=0, c3=0, c4=0, cs1=0, cs2=0;
+	switch(kind)
+	{
+		case 1:	c1=c2=1.;	break;
+		case 2:	c1=c2=2.;	c3=c4=-1.;	break;
+		case 3:	c1=c2=4./3.;	c3=c4=-1/3.;	cs1 = (-2./3.)*kdx;		cs2 = cs1/M_SQRT2; break;
+		case 4:	c1=4./(3.-2.*kdx);		c2=4./(3.-2.*kdx2);	c3=-1./(3.-2.*kdx);	c4=-1./(3.-2.*kdx2);	break;
+		case 5:	c1=4./(3.-dual(0,2)*kdx);	c2=4./(3.-dual(0,2)*kdx2);	c3=-1./(3.-dual(0,2)*kdx);	c4=-1./(3.-dual(0,2)*kdx2);	break;
+	}
+
+	std::vector<bndI> bh,bd,bs,be;
+	if(ncycl && !both)	// the same mask for all slices
+		mgl_collect_mask2d(bh,bd,bs,be, mask,0);
+	dual *res = new dual[nx*ny];
+	for(long k=0;k<nz;k++)
+	{
+		const long i0 = k*nx*ny;
+		dual *aa = d->a + i0;
+		if(ncycl && both)	// different mask for each slice
+			mgl_collect_mask2d(bh,bd,bs,be, mask,k);
+		double p0 = 0;
+		for(long i=0;i<nx*ny;i++)	p0 += norm(aa[i]);
+		if(p0==0)	continue;	// nothing to do since inifield is zero
+		for(long kk=0;kk<iter;kk++)
+		{
+#pragma omp parallel for collapse(2) schedule(static)
+			for(long j=1;j<ny-1;j++)	for(long i=1;i<nx-1;i++)
+			{
+				long ii = i+nx*j;
+				dual v = aa[ii];
+				res[ii] = (aa[ii-nx] + aa[ii-1] + 2.*v + aa[ii+1] + aa[ii+nx])/6. + k2*( nlin*norm(v) + pot->vthr(pth?ii+i0:ii) )*v;
+				res[ii] *= mask->vthr(both?ii+i0:ii);
+			}
+			if(ncycl)
+			{
+				for(size_t i=0;i<bh.size();i++)	// horizontal|vertical
+				{
+					const bndI &bb = bh[i];
+					res[bb.i0] = c1*res[bb.i1] + c3*res[bb.i2] + cs1;
+				}
+				for(size_t i=0;i<bd.size();i++)	// diagonal
+				{
+					const bndI &bb = bd[i];
+					//res[bb.i0] = c2*sqrt(res[bb.i1]*res[bb.i2]) + c4*res[bb.i3] + cs2;	// NOTE more correct but slower
+					res[bb.i0] = (0.5*c2)*(res[bb.i1]+res[bb.i2]) + c4*res[bb.i3] + cs2;
+				}
+				for(size_t i=0;i<bs.size();i++)	// average correct values
+				{
+					const bndI &bb = bs[i];
+					res[bb.i0] = 0.5*(res[bb.i1]+res[bb.i2]);
+				}
+				for(size_t i=0;i<be.size();i++)	// stupid case to exclude "dots"
+				{
+					const long i0 = be[i].i0;
+					res[i0] = (res[i0-nx]+res[i0-1]+res[i0+1]+res[i0+nx])/4.;
+				}
+			}
+			else
+			{
+#pragma omp parallel for schedule(static)
+				for(long j=1;j<ny-1;j++)
+				{
+					long ii = nx*j, ic = ii+nx-1;
+					dual v = aa[ii];
+					res[ii] = (aa[ic] + aa[ii-1] + 2.*v + aa[ii+1] + aa[ii+nx])/6. + k2*( nlin*norm(v) + pot->vthr(pth?ii+i0:ii) )*v;
+					res[ii] *= mask->vthr(both?ii+i0:ii);
+					v = aa[ic];
+					res[ic] = (aa[ic-nx] + aa[ic-1] + 2.*v + aa[ic+1] + aa[ii])/6. + k2*( nlin*norm(v) + pot->vthr(pth?ic+i0:ic) )*v;
+					res[ic] *= mask->vthr(both?ic+i0:ic);
+				}
+#pragma omp parallel for schedule(static)
+				for(long i=1;i<nx-1;i++)
+				{
+					long ii = i, ic = ii+nx*(ny-1);
+					dual v = aa[ii];
+					res[ii] = (aa[ic] + aa[ii-1] + 2.*v + aa[ii+1] + aa[ii+nx])/6. + k2*( nlin*norm(v) + pot->vthr(pth?ii+i0:ii) )*v;
+					res[ii] *= mask->vthr(both?ii+i0:ii);
+					v = aa[ic];
+					res[ic] = (aa[ic-nx] + aa[ic-1] + 2.*v + aa[ic+1] + aa[ii])/6. + k2*( nlin*norm(v) + pot->vthr(pth?ic+i0:ic) )*v;
+					res[ic] *= mask->vthr(both?ic+i0:ic);
+				}
+			}
+			double p=0;
+			for(long i=0;i<nx*ny;i++)	p += norm(res[i]);
+			p = sqrt(p0/p);
+			for(long i=0;i<nx*ny;i++)	aa[i] = res[i]*p;
+		}
+	}
+	delete []res;
+}
+//-----------------------------------------------------------------------------
+void mgl_mode3c(HADT d, HCDT mask, HCDT pot, const long nx, const long ny, const long nz, const int kind, const dual kdx, const long flag, long iter)
+{
+	// TODO
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_datac_mode(HADT d, HCDT mask, HCDT pot, const char *how, dual kdx, long iter)
+{
+	long nx = d->nx, ny = d->ny, nz = d->nz, flag = 0;
+	int kind = 0;
+	if(mglchr(how, '1'))	kind = 1;
+	else if(mglchr(how, '2'))	kind = 2;
+	else if(mglchr(how, 's'))	kind = 3;
+	else if(mglchr(how, 'r'))	kind = 4;
+	else if(mglchr(how, 'i'))	kind = 5;
+	else if(mglchr(how, 'a'))	kind = 6;
+
+	if(mglchr(how, 'c'))	flag |= 1;
+	if(mglchr(how, '+'))	flag |= 2;
+	if(mglchr(how, '-'))	flag |= 4;
+	bool isx=mglchr(how,'x');
+	bool isy=mglchr(how,'y');
+	bool isz=mglchr(how,'z');
+	mglDataV tmpM(nx,ny,nz), tmpP(nx,ny,nz);
+	tmpM.Fill(1,1);
+	if(!mask)	mask= &tmpM;
+	if(!pot)	pot = &tmpP;
+	if((!isx && !isy) || isz)	// whole array
+	{
+		if(nz>1)		mgl_mode3c(d,mask,pot,nx,ny,nz,kind,kdx,flag,iter);
+		else if(ny>1)	mgl_mode2c(d,mask,pot,nx,ny,nz,kind,kdx,flag,iter);
+		else if(nx>1)	mgl_mode1c(d,mask,pot,nx,ny*nz,kind,kdx,flag,iter);
+	}
+	else if(isy && nx>1 && ny>1)	// 2d array by nz times
+		mgl_mode2c(d,mask,pot,nx,ny,nz,kind,kdx,flag,iter);
+	else if(isx && nx>1)	// 1d array by ny*nz times
+		mgl_mode1c(d,mask,pot,nx,ny*nz,kind,kdx,flag,iter);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_datac_mode_(uintptr_t *d, uintptr_t *mask, uintptr_t *pot, const char *how, double *kdx, long *iter, int l)
+{
+	char *s=new char[l+1];	memcpy(s,how,l);	s[l]=0;
+	mgl_datac_mode(_DC_, _DA_(mask), _DA_(pot), s, *kdx, *iter);	delete []s;
+}
+//-----------------------------------------------------------------------------
