@@ -3,7 +3,7 @@
  * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
+ *   it under the terms of the GNU Lesser General Public License  as       *
  *   published by the Free Software Foundation; either version 3 of the    *
  *   License, or (at your option) any later version.                       *
  *                                                                         *
@@ -12,33 +12,31 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
+ *   You should have received a copy of the GNU Lesser General Public     *
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <set>
 #include <stdarg.h>
-#include <ctype.h>
 #include "mgl2/data.h"
 #include "mgl2/eval.h"
 #include "mgl2/thread.h"
 #include "interp.hpp"
 void MGL_NO_EXPORT mgl_txt_func(const mreal *x, mreal *dx, void *par);
-HMDT MGL_NO_EXPORT mglFormulaCalc(const char *str, const std::vector<mglDataA*> &head);
 //-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_data_trace(HCDT d)
 {
 	long nx=d->GetNx(),ny=d->GetNy(),nz=d->GetNz();
 	mglData *r=new mglData(nx);
 	if(ny>=nx && nz>=nx)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)	r->a[i] = d->v(i,i,i);
 	else if(ny>=nx)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)	r->a[i] = d->v(i,i);
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)	r->a[i] = d->v(i);
 	return r;
 }
@@ -107,10 +105,10 @@ HMDT MGL_EXPORT mgl_data_subdata_ext(HCDT d, HCDT xx, HCDT yy, HCDT zz)
 		if(!iy)	{	yy = &ty;	if(vy>=0)	ty.Fill(vy);	else ty.All();	}
 		if(!iz)	{	zz = &tz;	if(vz>=0)	tz.Fill(vz);	else tz.All();	}
 		r=new mglData(n,m,l);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i0=0;i0<n*m*l;i0++)
 		{
-			long x=long(0.5+xx->vthr(i0)), y=long(0.5+yy->vthr(i0)), z=long(0.5+zz->vthr(i0));
+			long x=long(floor(0.5+xx->vthr(i0))), y=long(floor(0.5+yy->vthr(i0))), z=long(floor(0.5+zz->vthr(i0)));
 			r->a[i0] = (x>=0 && x<nx && y>=0 && y<ny && z>=0 && z<nz)?d->v(x,y,z):NAN;
 		}
 	}
@@ -121,10 +119,10 @@ HMDT MGL_EXPORT mgl_data_subdata_ext(HCDT d, HCDT xx, HCDT yy, HCDT zz)
 		if(yy->GetNx()>1 || vy>=0)	m=yy->GetNx();	else	{	m=ny;	yy = &ty;	}
 		if(zz->GetNx()>1 || vz>=0)	l=zz->GetNx();	else	{	l=nz;	zz = &tz;	}
 		r=new mglData(n,m,l);
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<l;k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 		{
-			long x=long(0.5+xx->v(i)), y=long(0.5+yy->v(j)), z=long(0.5+zz->v(k));
+			long x=long(floor(0.5+xx->v(i))), y=long(floor(0.5+yy->v(j))), z=long(floor(0.5+zz->v(k)));
 			r->a[i+n*(j+m*k)] = (x>=0 && x<nx && y>=0 && y<ny && z>=0 && z<nz)?d->v(x,y,z):NAN;
 		}
 		if(m==1)	{	r->ny=r->nz;	r->nz=1;	}// "squeeze" dimensions
@@ -142,11 +140,11 @@ HMDT MGL_EXPORT mgl_data_subdata(HCDT d, long xx,long yy,long zz)
 	if(zz<0)	{	zz=0;	dz=1;	l=nz;	}
 	mglData *r=new mglData(n,m,l);
 	if(xx<nx && yy<ny && zz<nz)
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<l;k++)	for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
 			r->a[i+n*(j+m*k)] = d->v(xx+dx*i, yy+dy*j, zz+dz*k);
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<n*m*l;i++)	r->a[i] = NAN;
 	if(m==1)	{	r->ny=r->nz;	r->nz=1;	}// "squeeze" dimensions
 	if(n==1)	{	r->nx=r->ny;	r->ny=r->nz;	r->nz=1;	r->NewId();}
@@ -166,7 +164,7 @@ static void *mgl_resize(void *par)
 	const mreal *c=t->c;
 	HCDT dat = (HCDT)(t->v);
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
@@ -205,7 +203,7 @@ HMDT MGL_EXPORT mgl_data_combine(HCDT d1, HCDT d2)
 	if(n1==1)	{	n1=n2;	n2=d2->GetNy();	dim2 = false;	}
 	r->Create(nx,n1,n2);
 	if(dim2)	n1*=nx;	else	{	n2*=n1;	n1=nx;	}
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 	for(long j=0;j<n2;j++)	for(long i=0;i<n1;i++)
 		r->a[i+n1*j] = d1->vthr(i)*d2->vthr(j);
 	return r;
@@ -220,7 +218,7 @@ static void *mgl_sum_z(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -237,7 +235,7 @@ static void *mgl_sum_y(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -254,7 +252,7 @@ static void *mgl_sum_x(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -275,7 +273,7 @@ HMDT MGL_EXPORT mgl_data_sum(HCDT dat, const char *dir)
 	const mglData *d=dynamic_cast<const mglData *>(dat);
 	if(d)	memcpy(c,d->a,nx*ny*nz*sizeof(mreal));
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	c[i]=dat->vthr(i);
 
 	if(strchr(dir,'z') && nz>1)
@@ -309,7 +307,7 @@ static void *mgl_max_z(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -325,7 +323,7 @@ static void *mgl_max_y(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -341,7 +339,7 @@ static void *mgl_max_x(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -361,7 +359,7 @@ HMDT MGL_EXPORT mgl_data_max_dir(HCDT dat, const char *dir)
 	const mglData *d=dynamic_cast<const mglData *>(dat);
 	if(d)	memcpy(c,d->a,nx*ny*nz*sizeof(mreal));
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	c[i]=dat->vthr(i);
 
 	if(strchr(dir,'z') && nz>1)
@@ -388,6 +386,94 @@ uintptr_t MGL_EXPORT mgl_data_max_dir_(uintptr_t *d, const char *dir,int l)
 {	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
 	uintptr_t r=uintptr_t(mgl_data_max_dir(_DT_,s));	delete []s;	return r;	}
 //-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_data_first_dir(HCDT dat, const char *dir, double val)
+{
+	if(!dir || *dir==0)	return 0;
+	const long nx=dat->GetNx(),ny=dat->GetNy(),nz=dat->GetNz(), nn=nx*ny;
+	mglData *r=NULL;
+	if(mglchr(dir,'z') && nz>1)
+	{
+		r = new mglData(nx,ny);
+#pragma omp parallel for schedule(static)
+		for(long j=0;j<nn;j++)
+		{
+			long i;
+			for(i=0;i<nz;i++)	if(dat->vthr(j+nn*i)>val)	break;
+			r->a[j] = i/mreal(nz-1);
+		}
+	}
+	else if(mglchr(dir,'y') && ny>1)
+	{
+		r = new mglData(nx,nz);
+#pragma omp parallel for schedule(static)
+		for(long j=0;j<nx*nz;j++)
+		{
+			long i, i0 = (j%nx)+nn*(j/nx);
+			for(i=0;i<ny;i++)	if(dat->vthr(i0+nx*i)>val)	break;
+			r->a[j] = i/mreal(ny-1);
+		}
+	}
+	else if(mglchr(dir,'x') && nx>1)
+	{
+		r = new mglData(ny,nz);
+#pragma omp parallel for schedule(static)
+		for(long j=0;j<ny*nz;j++)
+		{
+			long i, i0 = j*nx;
+			for(i=0;i<nx;i++)	if(dat->vthr(i0+i)>val)	break;
+			r->a[j] = i/mreal(nx-1);
+		}
+	}
+	return r;
+}
+uintptr_t MGL_EXPORT mgl_data_first_dir_(uintptr_t *d, const char *dir, double *val,int l)
+{	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
+	uintptr_t r=uintptr_t(mgl_data_first_dir(_DT_,s,*val));	delete []s;	return r;	}
+//-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_data_last_dir(HCDT dat, const char *dir, double val)
+{
+	if(!dir || *dir==0)	return 0;
+	const long nx=dat->GetNx(),ny=dat->GetNy(),nz=dat->GetNz(), nn=nx*ny;
+	mglData *r=NULL;
+	if(mglchr(dir,'z') && nz>1)
+	{
+		r = new mglData(nx,ny);
+#pragma omp parallel for schedule(static)
+		for(long j=0;j<nn;j++)
+		{
+			long i;
+			for(i=nz-1;i>=0;i++)	if(dat->vthr(j+nn*i)>val)	break;
+			r->a[j] = i/mreal(nz-1);
+		}
+	}
+	else if(mglchr(dir,'y') && ny>1)
+	{
+		r = new mglData(nx,nz);
+#pragma omp parallel for schedule(static)
+		for(long j=0;j<nx*nz;j++)
+		{
+			long i, i0 = (j%nx)+nn*(j/nx);
+			for(i=ny-1;i>=0;i++)	if(dat->vthr(i0+nx*i)>val)	break;
+			r->a[j] = i/mreal(ny-1);
+		}
+	}
+	else if(mglchr(dir,'x') && nx>1)
+	{
+		r = new mglData(ny,nz);
+#pragma omp parallel for schedule(static)
+		for(long j=0;j<ny*nz;j++)
+		{
+			long i, i0 = j*nx;
+			for(i=nx-1;i>=0;i++)	if(dat->vthr(i0+i)>val)	break;
+			r->a[j] = i/mreal(nx-1);
+		}
+	}
+	return r;
+}
+uintptr_t MGL_EXPORT mgl_data_last_dir_(uintptr_t *d, const char *dir, double *val,int l)
+{	char *s=new char[l+1];	memcpy(s,dir,l);	s[l]=0;
+	uintptr_t r=uintptr_t(mgl_data_last_dir(_DT_,s,*val));	delete []s;	return r;	}
+//-----------------------------------------------------------------------------
 static void *mgl_min_z(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
@@ -395,7 +481,7 @@ static void *mgl_min_z(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -411,7 +497,7 @@ static void *mgl_min_y(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -427,7 +513,7 @@ static void *mgl_min_x(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -447,7 +533,7 @@ HMDT MGL_EXPORT mgl_data_min_dir(HCDT dat, const char *dir)
 	const mglData *d=dynamic_cast<const mglData *>(dat);
 	if(d)	memcpy(c,d->a,nx*ny*nz*sizeof(mreal));
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	c[i]=dat->vthr(i);
 
 	if(strchr(dir,'z') && nz>1)
@@ -565,7 +651,7 @@ HMDT MGL_EXPORT mgl_data_momentum(HCDT dat, char dir, const char *how)
 	if(dir=='x')
 	{
 		b=new mglData(nx);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx;i++)
 		{
 			mreal i1=0,i0=0;
@@ -580,7 +666,7 @@ HMDT MGL_EXPORT mgl_data_momentum(HCDT dat, char dir, const char *how)
 	if(dir=='y')
 	{
 		b=new mglData(ny);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<ny;i++)
 		{
 			mreal i1=0,i0=0;
@@ -596,7 +682,7 @@ HMDT MGL_EXPORT mgl_data_momentum(HCDT dat, char dir, const char *how)
 	{
 		long nn=nx*ny;
 		b=new mglData(nz);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nz;i++)
 		{
 			mreal i1=0,i0=0;
@@ -621,7 +707,7 @@ void MGL_EXPORT mgl_data_mul_dat(HMDT d, HCDT a)
 	if(mz==1 && my==1 && mx==1)
 	{
 		mreal v=a->v(0);
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 			d->a[i+nx*(j+ny*k)] *= v;
 	}
@@ -631,7 +717,7 @@ void MGL_EXPORT mgl_data_mul_dat(HMDT d, HCDT a)
 		if(nz*ny*nx==mz*my*mx)	{	n=nx*ny*nz;	m=1;	}
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] *= a->vthr(i);
 	}
 }
@@ -650,7 +736,7 @@ void MGL_EXPORT mgl_data_div_dat(HMDT d, HCDT a)
 	if(mz==1 && my==1 && mx==1)
 	{
 		mreal v=a->v(0);
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 			d->a[i+nx*(j+ny*k)] /= v;
 	}
@@ -660,7 +746,7 @@ void MGL_EXPORT mgl_data_div_dat(HMDT d, HCDT a)
 		if(nz*ny*nx==mz*my*mx)	{	n=nx*ny*nz;	m=1;	}
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] /= a->vthr(i);
 	}
 }
@@ -679,7 +765,7 @@ void MGL_EXPORT mgl_data_add_dat(HMDT d, HCDT a)
 	if(mz==1 && my==1 && mx==1)
 	{
 		mreal v=a->v(0);
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 			d->a[i+nx*(j+ny*k)] += v;
 	}
@@ -689,7 +775,7 @@ void MGL_EXPORT mgl_data_add_dat(HMDT d, HCDT a)
 		if(nz*ny*nx==mz*my*mx)	{	n=nx*ny*nz;	m=1;	}
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] += a->vthr(i);
 	}
 }
@@ -708,7 +794,7 @@ void MGL_EXPORT mgl_data_sub_dat(HMDT d, HCDT a)
 	if(mz==1 && my==1 && mx==1)
 	{
 		mreal v=a->v(0);
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 			d->a[i+nx*(j+ny*k)] -= v;
 	}
@@ -718,7 +804,7 @@ void MGL_EXPORT mgl_data_sub_dat(HMDT d, HCDT a)
 		if(nz*ny*nx==mz*my*mx)	{	n=nx*ny*nz;	m=1;	}
 		else if(ny*nx==my*mx)	{	n=nx*ny;	m=nz;	}
 		else if(nx==mx)			{	n=nx;	m=ny*nz;	}
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 		for(long k=0;k<m;k++)	for(long i=0;i<n;i++)	d->a[i+n*k] -= a->vthr(i);
 	}
 }
@@ -744,7 +830,7 @@ void static mgl_hist_p(mglThreadD *t,mreal *a)
 	long n=t[0].p[0];
 	memset(a,0,n*sizeof(mreal));
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=0;i<mglNumThr;i++)
 	{
@@ -762,7 +848,7 @@ static void *mgl_hist_1(void *par)
 	HCDT a = (HCDT)(t->b), c = (HCDT)(t->c);
 	const mreal *v=(const mreal *)t->v;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -779,37 +865,141 @@ static void *mgl_hist_2(void *par)
 {
 	mglThreadD *t=(mglThreadD *)par;
 	long nn=t->n, n = t->p[0];
-	long ns=t->p[1], nx=t->p[2], ny=t->p[3];
+	long nsub=t->p[1], nx=t->p[2], ny=t->p[3], ns=labs(nsub)+1;
 	mreal *b=new mreal[n], d=1./ns;
 	memset(b,0,n*sizeof(mreal));
 	HCDT a = (HCDT)(t->b), c = (HCDT)(t->c);
 	const mreal *v=(const mreal *)t->v;
-	bool sp = n>0;
-#if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
-#endif
-	for(long i=t->id;i<nn;i+=mglNumThr)
+	const long nsx=ns*nx, nsy=ns*ny, nxy=nsx*nsy;
+	if(nsub<0)	// spline
 	{
-		mreal x = d*(i%(nx*ns)), y = d*((i/(nx*ns))%(ny*ns)), z = d*(i/(nx*ny*ns*ns));
-		mreal f = sp ? a->value(x,y,z) : a->linear(x,y,z), w=1;
-		if(c)	w = sp ? c->value(x,y,z) : c->linear(x,y,z);
-		if(mgl_isnan(f) || mgl_isnan(w))	continue;
-		long k = long(n*(f-v[0])/(v[1]-v[0]));
-		if(k>=0 && k<n)
+		if(c)
+		{
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for schedule(static)
+#endif
+			for(long i=t->id;i<nn;i+=mglNumThr)
+			{
+				mreal x = d*(i%nsx), y = d*((i/nsx)%nsy), z = d*(i/nxy);
+				mreal f = a->value(x,y,z), w = c->value(x,y,z);
+				if(mgl_isnan(f) || mgl_isnan(w))	continue;
+				long k = long(n*(f-v[0])/(v[1]-v[0]));
+				if(k>=0 && k<n)
 #if !MGL_HAVE_PTHREAD
 #pragma omp critical(hist)
 #endif
-			b[k] += w * d*d*d;
+					b[k] += w * d*d*d;
+			}
+		}
+		else
+		{	
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for schedule(static)
+#endif
+			for(long i=t->id;i<nn;i+=mglNumThr)
+			{
+				mreal x = d*(i%nsx), y = d*((i/nsx)%nsy), z = d*(i/nxy);
+				mreal f = a->value(x,y,z);
+				if(mgl_isnan(f))	continue;
+				long k = long(n*(f-v[0])/(v[1]-v[0]));
+				if(k>=0 && k<n)
+#if !MGL_HAVE_PTHREAD
+#pragma omp critical(hist)
+#endif
+					b[k] += d*d*d;
+			}
+		}
+	}
+	else
+	{
+		if(c)
+		{
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for schedule(static)
+#endif
+			for(long i=t->id;i<nn;i+=mglNumThr)
+			{
+				mreal x = d*(i%nsx), y = d*((i/nsx)%nsy), z = d*(i/nxy);
+				mreal f = a->linear(x,y,z), w = c->linear(x,y,z);
+				if(mgl_isnan(f) || mgl_isnan(w))	continue;
+				long k = long(n*(f-v[0])/(v[1]-v[0]));
+				if(k>=0 && k<n)
+#if !MGL_HAVE_PTHREAD
+#pragma omp critical(hist)
+#endif
+					b[k] += w * d*d*d;
+			}
+		}
+		else
+		{	
+#if !MGL_HAVE_PTHREAD
+#pragma omp parallel for schedule(static)
+#endif
+			for(long i=t->id;i<nn;i+=mglNumThr)
+			{
+				mreal x = d*(i%nsx), y = d*((i/nsx)%nsy), z = d*(i/nxy);
+				mreal f = a->linear(x,y,z);
+				if(mgl_isnan(f))	continue;
+				long k = long(n*(f-v[0])/(v[1]-v[0]));
+				if(k>=0 && k<n)
+#if !MGL_HAVE_PTHREAD
+#pragma omp critical(hist)
+#endif
+					b[k] += d*d*d;
+			}
+		}
 	}
 	t->a = b;	return 0;
 }
+static void *mgl_hist_l(void *par)
+{
+	mglThreadD *t=(mglThreadD *)par;
+	long nn=t->n, n = t->p[0], nx1 = t->p[1]-1;
+	mreal *b=new mreal[n];
+	memset(b,0,n*sizeof(mreal));
+	HCDT a = (HCDT)(t->b), c = (HCDT)(t->c);
+	const mreal *v=(const mreal *)t->v;
+// #if !MGL_HAVE_PTHREAD
+// #pragma omp parallel for
+// #endif
+	for(long i=t->id;i<nn;i+=mglNumThr)
+	{
+		long ii = i%nx1+(nx1+1)*(i/nx1);
+		double d1 = n*(a->vthr(ii)-v[0])/(v[1]-v[0]), d, dd1, dd2;
+		double d2 = n*(a->vthr(ii+1)-v[0])/(v[1]-v[0]);
+		double c1 = c ? c->vthr(ii):1, c2 = c ? c->vthr(ii+1):1;
+		if(d2<d1)	{	d=d1;	d1=d2;	d2=d;	d=c1;	c1=c2;	c2=d;	}
+		long k1 = long(d1), k2 = long(d2);
+		k1 = k1>=0? k1:0;	k2 = k2<n?k2:n-1;
+		d = 1./(d2-d1);
+		dd1 = (1-(d1-k1))*d;
+		dd2 = (d2-k2)*d;
+		if(k1<k2)
+// #if !MGL_HAVE_PTHREAD
+// #pragma omp critical(hist)
+// #endif
+		{
+			b[k1] += c1*dd1;
+			b[k2] += c2*dd2;
+			for(long k=k1+1;k<k2;k++)
+				b[k] += (c1 + (k+0.5-d1)*d*(c2-c1))*d;
+		}
+		if(k1==k2)
+// #if !MGL_HAVE_PTHREAD
+// #pragma omp critical(hist)
+// #endif
+			b[k2] += c1;
+	}
+	t->a = b;	return 0;
+}
+//-----------------------------------------------------------------------------
 HMDT MGL_EXPORT mgl_data_hist(HCDT dat, long n, mreal v1, mreal v2, long nsub)
 {
 	if(n<2 || v1==v2)	return 0;
 	mglData *b=new mglData(n);
 	mreal v[2]={v1,v2};
 	long nx=dat->GetNx(), ny=dat->GetNy(), nz=dat->GetNz();
-	long ns=labs(nsub)+1, p[5]={n,ns,nx,ny,nz};
+	long ns=labs(nsub)+1, p[5]={n,nsub,nx,ny,nz};
 	if(nsub==0)	mglStartThread(mgl_hist_1,mgl_hist_p, nx*ny*nz, b->a,(const mreal *)dat,0,p,v);
 	else	mglStartThread(mgl_hist_2,mgl_hist_p, nx*ny*nz*ns*ns*ns, b->a,(const mreal *)dat,0,p,v);
 	return b;
@@ -822,16 +1012,30 @@ HMDT MGL_EXPORT mgl_data_hist_w(HCDT dat, HCDT weight, long n, mreal v1, mreal v
 	mreal v[2]={v1,v2};
 
 	long nx=dat->GetNx(), ny=dat->GetNy(), nz=dat->GetNz();
-	long ns=labs(nsub)+1, p[5]={n,ns,nx,ny,nz};
+	long ns=labs(nsub)+1, p[5]={n,nsub,nx,ny,nz};
 	if(nsub==0)	mglStartThread(mgl_hist_1,mgl_hist_p, nx*ny*nz, b->a,(const mreal *)dat,(const mreal *)weight,p,v);
 	else	mglStartThread(mgl_hist_2,mgl_hist_p, nx*ny*nz*ns*ns*ns, b->a,(const mreal *)dat,(const mreal *)weight,p,v);
 	return b;
 }
 //-----------------------------------------------------------------------------
+HMDT MGL_EXPORT mgl_data_hist_l(HCDT dat, HCDT weight, long n, mreal v1, mreal v2)
+{
+	if(n<2 || v1==v2)	return 0;
+	mglData *b=new mglData(n);
+	mreal v[2]={v1,v2};
+
+	long nx=dat->GetNx(), ny=dat->GetNy(), nz=dat->GetNz();
+	long p[5]={n,nx,ny,nz};
+	mglStartThread(mgl_hist_l,mgl_hist_p, (nx-1)*ny*nz, b->a,(const mreal *)dat,(const mreal *)weight,p,v);
+	return b;
+}
+//-----------------------------------------------------------------------------
 uintptr_t MGL_EXPORT mgl_data_hist_(uintptr_t *d, int *n, mreal *v1, mreal *v2, int *nsub)
 {	return uintptr_t(mgl_data_hist(_DT_,*n,*v1,*v2,*nsub));	}
 uintptr_t MGL_EXPORT mgl_data_hist_w_(uintptr_t *d, uintptr_t *w, int *n, mreal *v1, mreal *v2, int *nsub)
 {	return uintptr_t(mgl_data_hist_w(_DT_,_DA_(w),*n,*v1,*v2,*nsub));	}
+uintptr_t MGL_EXPORT mgl_data_hist_l_(uintptr_t *d, uintptr_t *w, int *n, mreal *v1, mreal *v2)
+{	return uintptr_t(mgl_data_hist_l(_DT_,_DA_(w),*n,*v1,*v2));	}
 //-----------------------------------------------------------------------------
 long MGL_NO_EXPORT mgl_idx_var;
 int MGL_LOCAL_PURE mgl_cmd_idx(const void *a, const void *b)
@@ -878,7 +1082,7 @@ int static mgl_root(const gsl_vector *x, void *params, gsl_vector *f)
 		if(mgl_isbad(p->f[i]))	ok=false;	}
 	return ok?GSL_SUCCESS:GSL_FAILURE;
 }
-bool MGL_EXPORT mgl_find_roots(size_t n, void (*func)(const mreal *x, mreal *f, void *par), mreal *x0, void *par)
+int MGL_EXPORT mgl_find_roots(size_t n, void (*func)(const mreal *x, mreal *f, void *par), mreal *x0, void *par)
 {
 	for(size_t i=0;i<n;i++)	if(mgl_isbad(x0[i]))	return false;
 	mglRoots pp;
@@ -905,7 +1109,7 @@ bool MGL_EXPORT mgl_find_roots(size_t n, void (*func)(const mreal *x, mreal *f,
 	return status==GSL_SUCCESS;
 }
 #else
-bool MGL_EXPORT mgl_find_roots(size_t , void (*)(const mreal *, mreal *, void *), mreal *, void *)
+int MGL_EXPORT mgl_find_roots(size_t n, void (*func)(const mreal *x, mreal *f, void *par), mreal *x0, void *par)
 {	return false;	}
 #endif // MGL_HAVE_GSL
 //-----------------------------------------------------------------------------
@@ -914,7 +1118,7 @@ HMDT MGL_EXPORT mgl_find_roots_txt(const char *func, const char *vars, HCDT ini)
 	if(!vars || !(*vars) || !func || !ini)	return 0;
 	mglEqTxT par;
 	par.var=vars;	par.FillReal(func);
-	size_t n = par.str.size();
+	size_t n = par.str.size();	par.n = n;
 	if(ini->GetNx()!=long(n))	return 0;
 	mreal *xx = new mreal[n];
 	mglData *res = new mglData(ini);
@@ -968,7 +1172,7 @@ HMDT MGL_EXPORT mgl_data_roots(const char *func, HCDT ini, char var)
 	mglFormula eq(func);
 	mglFuncV f;	f.eq = &eq;	f.var = var;
 	long n = res->nx*res->ny*res->nz;
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=0;i<n;i++)
 		res->a[i] = mgl_find_root(mgl_funcv,res->a[i],&f);
 	return res;
@@ -997,7 +1201,7 @@ static void *mgl_pulse_z(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -1037,7 +1241,7 @@ static void *mgl_pulse_y(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -1078,7 +1282,7 @@ static void *mgl_pulse_x(void *par)
 	mreal *b=t->a;
 	const mreal *a=t->b;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i=t->id;i<nn;i+=mglNumThr)
 	{
@@ -1121,7 +1325,7 @@ HMDT MGL_EXPORT mgl_data_pulse(HCDT dat, char dir)
 	const mglData *d=dynamic_cast<const mglData *>(dat);
 	if(d)	memcpy(c,d->a,nx*ny*nz*sizeof(mreal));
 	else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=0;i<nx*ny*nz;i++)	c[i]=dat->vthr(i);
 
 	if(dir=='z' && nz>1)
@@ -1188,7 +1392,7 @@ HMDT MGL_EXPORT mgl_data_section(HCDT dat, HCDT ids, char dir, mreal val)
 		for(size_t s=0;s<ns;s++)
 		{
 			long pp = ps[s];
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<nz;k++)	for(long j=0;j<ls[s];j++)	for(long i=0;i<nx;i++)
 				r->a[i+nx*(sh+j+nl*k)] = dat->v(i,pp+j,k);
 			sh += ls[s];
@@ -1201,7 +1405,7 @@ HMDT MGL_EXPORT mgl_data_section(HCDT dat, HCDT ids, char dir, mreal val)
 		for(size_t s=0;s<ns;s++)
 		{
 			long pp = ps[s];
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<ls[s];i++)
 				r->a[sh+i+nl*(j+ny*k)] = dat->v(pp+i,j,k);
 			sh += ls[s];
@@ -1214,7 +1418,7 @@ HMDT MGL_EXPORT mgl_data_section(HCDT dat, HCDT ids, char dir, mreal val)
 		for(size_t s=0;s<ns;s++)
 		{
 			long pp = ps[s];
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 			for(long k=0;k<ls[s];k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 				r->a[i+nx*(j+ny*(sh+k))] = dat->v(i,j,pp+k);
 			sh += ls[s];
@@ -1304,3 +1508,271 @@ void MGL_EXPORT mgl_data_connect_r(HMDT a, HMDT b)
 void MGL_EXPORT mgl_data_connect_r_(uintptr_t *a, uintptr_t *b)
 {	mgl_data_connect_r(_DM_(a),_DM_(b));	}
 //-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_data_keep(HMDT dat, const char *how, long i, long j)
+{
+	const long nx = dat->GetNx(), ny = dat->GetNy(), nz = dat->GetNz();
+	const bool ampl = mglchr(how, 'a');
+	if(mglchr(how,'z'))
+	{
+		long ix = i, iy = j;
+		if(ix<0 || ix>=nx)	ix = 0;
+		if(iy<0 || iy>=ny)	iy = 0;
+		const long i0 = ix+nx*iy, nn = nx*ny;
+		const mreal v0 = dat->a[i0];
+		for(long k=0;k<nz;k++)
+		{
+			mreal v = dat->a[i0+nn*k];
+			mreal f = ampl? v0/v : (v*v0>0?1:-1);
+			for(long ii=0;ii<nn;ii++)	dat->a[ii+nn*k] *= f;
+		}
+	}
+	else if(mglchr(how,'x'))
+	{
+		long iy = i, iz = j;
+		if(iz<0 || iz>=nz)	iz = 0;
+		if(iy<0 || iy>=ny)	iy = 0;
+		const long i0 = nx*(iy+ny*iz), nn = ny*nz;
+		const mreal v0 = dat->a[i0];
+		for(long k=0;k<nx;k++)
+		{
+			mreal v = dat->a[i0+k];
+			mreal f = ampl? v0/v : (v*v0>0?1:-1);
+			for(long ii=0;ii<nn;ii++)	dat->a[k+nx*ii] *= f;
+		}
+	}
+	else	// default is "y"
+	{
+		long ix = i, iz = j;
+		if(ix<0 || ix>=nx)	ix = 0;
+		if(iz<0 || iz>=nz)	iz = 0;
+		const long i0 = ix+nx*ny*iz;
+		const mreal v0 = dat->a[i0];
+		for(long k=0;k<ny;k++)
+		{
+			mreal v = dat->a[i0+nx*k];
+			mreal f = ampl? v0/v : (v*v0>0?1:-1);
+			for(long ii=0;ii<nz;ii++)	for(long jj=0;jj<nx;jj++)	dat->a[jj+nx*(k+ny*ii)] *= f;
+		}
+	}
+}
+void MGL_EXPORT mgl_data_keep_(uintptr_t *d, const char *how, long *i, long *j, int l)
+{	char *s=new char[l+1];	memcpy(s,how,l);	s[l]=0;
+	mgl_data_keep(_DT_, s, *i, *j);	delete []s;	}
+//-----------------------------------------------------------------------------
+struct bndI	// NOTE: defined in complex_ex.cpp
+{	long i0, i1, i2, i3;
+	bndI(long j0,long j1,long j2=0,long j3=0):i0(j0),i1(j1),i2(j2),i3(j3)	{}
+};
+//-----------------------------------------------------------------------------
+void mgl_collect_mask1d(std::vector<bndI> &bnd, HCDT mask, long j);
+void mgl_mode1r(HMDT d, HCDT mask, HCDT pot, const long nx, const long nn, const int kind, const double kdx, const long flag, long iter)
+{
+	if(mask->GetNx()!=nx || pot->GetNx()!=nx)	return;
+	if(iter<1)	iter = nx*nx/(1-iter);
+	bool both = mask->GetNN() == nx*nn;
+	bool pth = pot->GetNN() == nx*nn;
+	bool ncycl = !(flag&1);
+	double nlin=0;
+	if(flag&2)	nlin = 1;
+	if(flag&4)	nlin =-1;
+	const double k2 = kdx*kdx;
+	double c1=0, c2=0, cs=0;
+	switch(kind)
+	{
+		case 1:	c1=1.;	break;
+		case 2:	c1=2.;	c2 = -1.;	break;
+		case 3:	c1=4./3.;	c2 = -1/3.;	cs = (-2./3.)*kdx;	break;
+		case 4:	c1=4./(3.-2.*kdx);		c2=-1./(3.-2.*kdx);	break;
+	}
+
+	std::vector<bndI> bnd;
+	if(ncycl && !both)	// the same mask for all slices
+		mgl_collect_mask1d(bnd, mask,0);
+	mreal *res = new mreal[nx];
+	for(long j=0;j<nn;j++)
+	{
+		mreal *aa = d->a + j*nx;
+		long i0 = j*nx;
+		if(ncycl && both)	// different mask for each slice
+			mgl_collect_mask1d(bnd, mask,j);
+		double p0 = 0;
+		for(long i=0;i<nx;i++)	p0 += aa[i]*aa[i];
+		if(p0==0)	continue;	// nothing to do since inifield is zero
+		for(long kk=0;kk<iter;kk++)
+		{
+#pragma omp parallel for schedule(static)
+			for(long i=1;i<nx-1;i++)
+			{
+				double v = aa[i];
+				res[i] = (aa[i-1] + v + aa[i+1])/3. + k2*( nlin*v*v + pot->vthr(pth?i+i0:i) )*v;
+				res[i] *= mask->vthr(both?i+i0:i);
+			}
+			if(ncycl)	for(size_t i=0;i<bnd.size();i++)
+			{
+				const bndI &bb = bnd[i];
+				res[bb.i0] = c1*res[bb.i1] + c2*res[bb.i2] + cs;
+			}
+			else
+			{
+				double v = aa[0];
+				res[0] = (aa[nx-1] + v + aa[1])/3. + k2*( nlin*v*v + pot->vthr(pth?i0:0) )*v;
+				res[0] *= mask->vthr(both?i0:0);
+				const long i = nx-1;	v = aa[i];
+				res[i] = (aa[i-1] + v + aa[0])/3. + k2*( nlin*v*v + pot->vthr(pth?i+i0:i) )*v;
+				res[i] *= mask->vthr(both?i+i0:i);
+			}
+			double p=0;
+			for(long i=0;i<nx;i++)	p += res[i]*res[i];
+			p = sqrt(p0/p);
+			for(long i=0;i<nx;i++)	aa[i] = res[i]*p;
+		}
+	}
+	delete []res;
+}
+//-----------------------------------------------------------------------------
+void mgl_collect_mask2d(std::vector<bndI> &h, std::vector<bndI> &d, std::vector<bndI> &s, std::vector<bndI> &e, HCDT mask, long k);
+void mgl_mode2r(HMDT d, HCDT mask, HCDT pot, const long nx, const long ny, const long nz, const int kind, const double kdx, const long flag, long iter)
+{
+	if(mask->GetNx()!=nx || pot->GetNx()!=nx || mask->GetNy()!=ny || pot->GetNy()!=ny)	return;
+	if(iter<1)	iter = (nx>=ny?nx*nx:ny*ny)/(1-iter);
+	bool both = mask->GetNN() == nx*ny*nz;
+	bool pth = pot->GetNN() == nx*ny*nz;
+	bool ncycl = !(flag&1);
+	double nlin=0;
+	if(flag&2)	nlin = 1;
+	if(flag&4)	nlin =-1;
+	const double kdx2 = kdx/M_SQRT2;
+	const double k2 = kdx*kdx;
+	double c1=0, c2=0, c3=0, c4=0, cs1=0, cs2=0;
+	switch(kind)
+	{
+		case 1:	c1=c2=1.;	break;
+		case 2:	c1=c2=2.;	c3=c4=-1.;	break;
+		case 3:	c1=c2=4./3.;	c3=c4=-1/3.;	cs1 = (-2./3.)*kdx;		cs2 = cs1/M_SQRT2; break;
+		case 4:	c1=4./(3.-2.*kdx);		c2=4./(3.-2.*kdx2);	c3=-1./(3.-2.*kdx);	c4=-1./(3.-2.*kdx2);	break;
+	}
+
+	std::vector<bndI> bh,bd,bs,be;
+	if(ncycl && !both)	// the same mask for all slices
+		mgl_collect_mask2d(bh,bd,bs,be, mask,0);
+	mreal *res = new mreal[nx*ny];
+	for(long k=0;k<nz;k++)
+	{
+		const long i0 = k*nx*ny;
+		mreal *aa = d->a + i0;
+		if(ncycl && both)	// different mask for each slice
+			mgl_collect_mask2d(bh,bd,bs,be, mask,k);
+		double p0 = 0;
+		for(long i=0;i<nx*ny;i++)	p0 += aa[i]*aa[i];
+		if(p0==0)	continue;	// nothing to do since inifield is zero
+		for(long kk=0;kk<iter;kk++)
+		{
+#pragma omp parallel for collapse(2) schedule(static)
+			for(long j=1;j<ny-1;j++)	for(long i=1;i<nx-1;i++)
+			{
+				long ii = i+nx*j;
+				double v = aa[ii];
+				res[ii] = (aa[ii-nx] + aa[ii-1] + 2.*v + aa[ii+1] + aa[ii+nx])/6. + k2*( nlin*v*v + pot->vthr(pth?ii+i0:ii) )*v;
+				res[ii] *= mask->vthr(both?ii+i0:ii);
+			}
+			if(ncycl)
+			{
+				for(size_t i=0;i<bh.size();i++)	// horizontal|vertical
+				{
+					const bndI &bb = bh[i];
+					res[bb.i0] = c1*res[bb.i1] + c3*res[bb.i2] + cs1;
+				}
+				for(size_t i=0;i<bd.size();i++)	// diagonal
+				{
+					const bndI &bb = bd[i];
+					//res[bb.i0] = c2*sqrt(res[bb.i1]*res[bb.i2]) + c4*res[bb.i3] + cs2;	// NOTE more correct but slower
+					res[bb.i0] = (0.5*c2)*(res[bb.i1]+res[bb.i2]) + c4*res[bb.i3] + cs2;
+				}
+				for(size_t i=0;i<bs.size();i++)	// average correct values
+				{
+					const bndI &bb = bs[i];
+					res[bb.i0] = 0.5*(res[bb.i1]+res[bb.i2]);
+				}
+				for(size_t i=0;i<be.size();i++)	// stupid case to exclude "dots"
+				{
+					const long i0 = be[i].i0;
+					res[i0] = (res[i0-nx]+res[i0-1]+res[i0+1]+res[i0+nx])/4.;
+				}
+			}
+			else
+			{
+#pragma omp parallel for schedule(static)
+				for(long j=1;j<ny-1;j++)
+				{
+					long ii = nx*j, ic = ii+nx-1;
+					double v = aa[ii];
+					res[ii] = (aa[ic] + aa[ii-1] + 2.*v + aa[ii+1] + aa[ii+nx])/6. + k2*( nlin*v*v + pot->vthr(pth?ii+i0:ii) )*v;
+					res[ii] *= mask->vthr(both?ii+i0:ii);
+					v = aa[ic];
+					res[ic] = (aa[ic-nx] + aa[ic-1] + 2.*v + aa[ic+1] + aa[ii])/6. + k2*( nlin*v*v + pot->vthr(pth?ic+i0:ic) )*v;
+					res[ic] *= mask->vthr(both?ic+i0:ic);
+				}
+#pragma omp parallel for schedule(static)
+				for(long i=1;i<nx-1;i++)
+				{
+					long ii = i, ic = ii+nx*(ny-1);
+					double v = aa[ii];
+					res[ii] = (aa[ic] + aa[ii-1] + 2.*v + aa[ii+1] + aa[ii+nx])/6. + k2*( nlin*v*v + pot->vthr(pth?ii+i0:ii) )*v;
+					res[ii] *= mask->vthr(both?ii+i0:ii);
+					v = aa[ic];
+					res[ic] = (aa[ic-nx] + aa[ic-1] + 2.*v + aa[ic+1] + aa[ii])/6. + k2*( nlin*v*v + pot->vthr(pth?ic+i0:ic) )*v;
+					res[ic] *= mask->vthr(both?ic+i0:ic);
+				}
+			}
+			double p=0;
+			for(long i=0;i<nx*ny;i++)	p += res[i]*res[i];
+			p = sqrt(p0/p);
+			for(long i=0;i<nx*ny;i++)	aa[i] = res[i]*p;
+		}
+	}
+	delete []res;
+}
+//-----------------------------------------------------------------------------
+void mgl_mode3r(HMDT d, HCDT mask, HCDT pot, const long nx, const long ny, const long nz, const int kind, const double kdx, const long flag, long iter)
+{	// TODO
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_data_mode(HMDT d, HCDT mask, HCDT pot, const char *how, double kdx, long iter)
+{
+	long nx = d->nx, ny = d->ny, nz = d->nz, flag = 0;
+	int kind = 0;
+	if(mglchr(how, '1'))	kind = 1;
+	else if(mglchr(how, '2'))	kind = 2;
+	else if(mglchr(how, 's'))	kind = 3;
+	else if(mglchr(how, 'r'))	kind = 4;
+	else if(mglchr(how, 'i'))	kind = 5;
+	else if(mglchr(how, 'a'))	kind = 6;
+
+	if(mglchr(how, 'c'))	flag |= 1;
+	if(mglchr(how, '+'))	flag |= 2;
+	if(mglchr(how, '-'))	flag |= 4;
+	bool isx=mglchr(how,'x');
+	bool isy=mglchr(how,'y');
+	bool isz=mglchr(how,'z');
+	mglDataV tmpM(nx,ny,nz), tmpP(nx,ny,nz);
+	tmpM.Fill(1,1);
+	if(!mask)	mask= &tmpM;
+	if(!pot)	pot = &tmpP;
+	if((!isx && !isy) || isz)	// whole array
+	{
+		if(nz>1)		mgl_mode3r(d,mask,pot,nx,ny,nz,kind,kdx,flag,iter);
+		else if(ny>1)	mgl_mode2r(d,mask,pot,nx,ny,nz,kind,kdx,flag,iter);
+		else if(nx>1)	mgl_mode1r(d,mask,pot,nx,ny*nz,kind,kdx,flag,iter);
+	}
+	else if(isy && nx>1 && ny>1)	// 2d array by nz times
+		mgl_mode2r(d,mask,pot,nx,ny,nz,kind,kdx,flag,iter);
+	else if(isx && nx>1)	// 1d array by ny*nz times
+		mgl_mode1r(d,mask,pot,nx,ny*nz,kind,kdx,flag,iter);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_data_mode_(uintptr_t *d, uintptr_t *mask, uintptr_t *pot, const char *how, double *kdx, long *iter, int l)
+{
+	char *s=new char[l+1];	memcpy(s,how,l);	s[l]=0;
+	mgl_data_mode(_DT_, _DA_(mask), _DA_(pot), s, *kdx, *iter);	delete []s;
+}
+//-----------------------------------------------------------------------------
