@@ -69,7 +69,7 @@ void mglFromStr(HMDT d,char *buf,long NX,long NY,long NZ)
 	{	lines.push_back(s+1);	*s = 0;	s++;	}
 	numbs.resize(lines.size());
 	long nl = long(lines.size());
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long k=0;k<nl;k++)
 	{
 		char *b = lines[k];
@@ -114,7 +114,7 @@ void MGL_EXPORT mgl_data_set(HMDT d, HCDT a)
 	if(dd)	// this one should be much faster
 		memcpy(d->a, dd->a, d->nx*d->ny*d->nz*sizeof(mreal));
 	else	// very inefficient!!!
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<d->nz;k++)	for(long j=0;j<d->ny;j++)	for(long i=0;i<d->nx;i++)
 			d->a[i+d->nx*(j+d->ny*k)] = a->v(i,j,k);
 }
@@ -138,7 +138,7 @@ void MGL_EXPORT mgl_data_set_vector(HMDT d, gsl_vector *v)
 #if MGL_HAVE_GSL
 	if(!v || v->size<1)	return;
 	mgl_data_create(d, v->size,1,1);
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=0;i<d->nx;i++)	d->a[i] = v->data[i*v->stride];
 #endif
 }
@@ -148,7 +148,7 @@ void MGL_EXPORT mgl_data_set_matrix(HMDT d, gsl_matrix *m)
 #if MGL_HAVE_GSL
 	if(!m || m->size1<1 || m->size2<1)	return;
 	mgl_data_create(d, m->size1,m->size2,1);
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 	for(long j=0;j<d->ny;j++)	for(long i=0;i<d->nx;i++)
 		d->a[i+j*d->nx] = m->data[i * m->tda + j];
 #endif
@@ -159,7 +159,7 @@ void MGL_EXPORT mgl_data_set_float(HMDT d, const float *A,long NX,long NY,long N
 	if(NX<=0 || NY<=0 || NZ<=0)	return;
 	mgl_data_create(d, NX,NY,NZ);	if(!A)	return;
 #if MGL_USE_DOUBLE
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=0;i<NX*NY*NZ;i++)	d->a[i] = A[i];
 #else
 	memcpy(d->a,A,NX*NY*NZ*sizeof(float));
@@ -173,7 +173,7 @@ void MGL_EXPORT mgl_data_set_double(HMDT d, const double *A,long NX,long NY,long
 #if MGL_USE_DOUBLE
 	memcpy(d->a,A,NX*NY*NZ*sizeof(double));
 #else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=0;i<NX*NY*NZ;i++)	d->a[i] = A[i];
 #endif
 }
@@ -183,7 +183,7 @@ void MGL_EXPORT mgl_data_set_float2(HMDT d, float const * const *A,long N1,long
 	if(N1<=0 || N2<=0)	return;
 	mgl_data_create(d, N2,N1,1);	if(!A)	return;
 #if MGL_USE_DOUBLE
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	d->a[j+i*N2] = A[i][j];
 #else
 #pragma omp parallel for
@@ -199,7 +199,7 @@ void MGL_EXPORT mgl_data_set_double2(HMDT d, double const *const *A,long N1,long
 #pragma omp parallel for
 	for(long i=0;i<N1;i++)	memcpy(d->a+i*N2,A[i],N2*sizeof(double));
 #else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	d->a[j+i*N2] = A[i][j];
 #endif
 }
@@ -209,11 +209,11 @@ void MGL_EXPORT mgl_data_set_float3(HMDT d, float const * const * const *A,long
 	if(N1<=0 || N2<=0 || N3<=0)	return;
 	mgl_data_create(d, N3,N2,N1);	if(!A)	return;
 #if MGL_USE_DOUBLE
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	for(long k=0;k<N3;k++)
 		d->a[k+N3*(j+i*N2)] = A[i][j][k];
 #else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)
 		memcpy(d->a+N3*(j+i*N2),A[i][j],N3*sizeof(float));
 #endif
@@ -224,11 +224,11 @@ void MGL_EXPORT mgl_data_set_double3(HMDT d, double const * const * const *A,lon
 	if(N1<=0 || N2<=0 || N3<=0)	return;
 	mgl_data_create(d, N3,N2,N1);	if(!A)	return;
 #if MGL_USE_DOUBLE
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)
 		memcpy(d->a+N3*(j+i*N2),A[i][j],N3*sizeof(double));
 #else
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 	for(long i=0;i<N1;i++)	for(long j=0;j<N2;j++)	for(long k=0;k<N3;k++)
 		d->a[k+N3*(j+i*N2)] = A[i][j][k];
 #endif
@@ -822,7 +822,7 @@ long MGL_EXPORT mgl_data_max_first(HCDT d, char dir, long from, long *p1, long *
 	{
 		for(long i=from+1;i<n-1;i++)
 		{
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long i1=0;i1<n1;i1++)	for(long i2=0;i2<n2;i2++)
 			{
 				long ii=i*dd+i1*d1+i2*d2;
@@ -926,26 +926,20 @@ void MGL_EXPORT mgl_data_fill(HMDT d, mreal x1,mreal x2,char dir)
 	if(dir=='x')
 	{
 		mreal dx = d->nx>1 ? (x2-x1)/(d->nx-1):0;
-#pragma omp parallel for collapse(2)
-		for(long j=0;j<ny*nz;j++)	for(long i=1;i<nx;i++)	d->a[i+nx*j] = x1+dx*i;
-#pragma omp parallel for
-		for(long j=0;j<ny*nz;j++)	d->a[nx*j] = x1;
+#pragma omp parallel for collapse(2) schedule(static)
+		for(long j=0;j<ny*nz;j++)	for(long i=0;i<nx;i++)	d->a[i+nx*j] = x1+dx*i;
 	}
 	if(dir=='y')
 	{
 		mreal dx = d->ny>1 ? (x2-x1)/(d->ny-1):0;
-#pragma omp parallel for collapse(3)
-		for(long k=0;k<nz;k++)	for(long j=1;j<ny;j++)	for(long i=0;i<nx;i++)	d->a[i+nx*(j+ny*k)] = x1+dx*j;
-#pragma omp parallel for collapse(2)
-		for(long j=0;j<nz;j++)	for(long i=0;i<nx;i++)	d->a[i+nx*ny*j] = x1;
+#pragma omp parallel for collapse(3) schedule(static)
+		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)	d->a[i+nx*(j+ny*k)] = x1+dx*j;
 	}
 	if(dir=='z')
 	{
 		mreal dx = d->nz>1 ? (x2-x1)/(d->nz-1):0;
-#pragma omp parallel for collapse(2)
-		for(long j=1;j<nz;j++)	for(long i=0;i<nx*ny;i++)	d->a[i+nx*ny*j] = x1+dx*j;
-#pragma omp parallel for
-		for(long j=0;j<nx*ny;j++)	d->a[j] = x1;
+#pragma omp parallel for collapse(2) schedule(static)
+		for(long j=0;j<nz;j++)	for(long i=0;i<nx*ny;i++)	d->a[i+nx*ny*j] = x1+dx*j;
 	}
 }
 void MGL_EXPORT mgl_data_fill_(uintptr_t *d, mreal *x1,mreal *x2,const char *dir,int)
@@ -966,7 +960,7 @@ void MGL_EXPORT mgl_data_norm(HMDT d, mreal v1,mreal v2,int sym,long dim)
 		a2 = -a1>a2 ? -a1:a2;	a1 = -a2;
 	}
 	v2 = (v2-v1)/(a2-a1);	v1 = v1-a1*v2;
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=s;i<nn;i++)	a[i] = v1 + v2*a[i];
 }
 void MGL_EXPORT mgl_data_norm_(uintptr_t *d, mreal *v1,mreal *v2,int *sym,int *dim)
@@ -988,11 +982,11 @@ void MGL_EXPORT mgl_data_squeeze(HMDT d, long rx,long ry,long rz,long smooth)
 	kx = 1+(nx-1)/rx;	ky = 1+(ny-1)/ry;	kz = 1+(nz-1)/rz;
 	b = new mreal[kx*ky*kz];
 	if(!smooth)
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<kz;k++)	for(long j=0;j<ky;j++)	for(long i=0;i<kx;i++)
 			b[i+kx*(j+ky*k)] = d->a[i*rx+nx*(j*ry+ny*rz*k)];
 	else
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<kz;k++)	for(long j=0;j<ky;j++)	for(long i=0;i<kx;i++)
 		{
 			long dx,dy,dz,i1,j1,k1;
@@ -1022,10 +1016,10 @@ void MGL_EXPORT mgl_data_extend(HMDT d, long n1, long n2)
 		mx = nx;	my = ny>1?ny:n1;	mz = ny>1 ? n1 : n2;
 		b = new mreal[mx*my*mz];
 		if(ny>1)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i=0;i<n1;i++)	memcpy(b+i*nx*ny, d->a, nx*ny*sizeof(mreal));
 		else
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i=0;i<n1*n2;i++)	memcpy(b+i*nx, d->a, nx*sizeof(mreal));
 	}
 	else
@@ -1034,15 +1028,15 @@ void MGL_EXPORT mgl_data_extend(HMDT d, long n1, long n2)
 		if(n2>0 && ny==1)	mz = n2;
 		b = new mreal[mx*my*mz];
 		if(n2<0)
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long j=0;j<nx;j++)	for(long i=0;i<mx*my;i++)
 				b[i+mx*my*j] = d->a[j];
 		else
-#pragma omp parallel for collapse(2)
+#pragma omp parallel for collapse(2) schedule(static)
 			for(long j=0;j<nx*ny;j++)	for(long i=0;i<mx;i++)
 				b[i+mx*j] = d->a[j];
 		if(n2>0 && ny==1)
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 			for(long i=0;i<n2;i++)
 				memcpy(b+i*mx*my, d->a, mx*my*sizeof(mreal));
 	}
@@ -1059,35 +1053,35 @@ void MGL_EXPORT mgl_data_transpose(HMDT d, const char *dim)
 	mreal *b=new mreal[nx*ny*nz], *a=d->a;
 	if(!strcmp(dim,"xzy") || !strcmp(dim,"zy"))
 	{
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
 			b[i+nx*(k+nz*j)] = a[i+nx*(j+ny*k)];
 		n=nz;	nz=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"yxz") || !strcmp(dim,"yx"))
 	{
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 			b[j+ny*(i+nx*k)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"yzx"))
 	{
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 			b[j+ny*(k+nz*i)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=ny;	ny=nz;	nz=n;
 	}
 	else if(!strcmp(dim,"zxy"))
 	{
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)
 			b[k+nz*(i+nx*j)] = a[i+nx*(j+ny*k)];
 		n=nx;	nx=nz;	nz=ny;	ny=n;
 	}
 	else if(!strcmp(dim,"zyx") || !strcmp(dim,"zx"))
 	{
-#pragma omp parallel for collapse(3)
+#pragma omp parallel for collapse(3) schedule(static)
 		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)	for(long k=0;k<nz;k++)
 			b[k+nz*(j+ny*i)] = a[i+nx*(j+ny*k)];
 		n=nz;	nz=nx;	nx=n;
@@ -1110,7 +1104,7 @@ static void *mgl_modify(void *par)
 	const mreal *v=t->b, *w=t->c;
 	dx=nx>1?1/(nx-1.):0;	dy=ny>1?1/(ny-1.):0;	dz=nz>1?1/(nz-1.):0;
 #if !MGL_HAVE_PTHREAD
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 #endif
 	for(long i0=t->id;i0<t->n;i0+=mglNumThr)
 	{
@@ -1525,7 +1519,7 @@ void MGL_EXPORT mgl_data_limit(HMDT d, mreal v)
 {
 	long n = d->GetNN();
 	mreal *a = d->a;
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=0;i<n;i++)
 	{	mreal b = fabs(a[i]);	if(b>v)	a[i] *= v/b;	}
 }
@@ -1540,7 +1534,7 @@ void MGL_EXPORT mgl_data_coil(HMDT d, mreal v1, mreal v2, int sep)
 	mreal *a = d->a, dv = v2-v1;
 	if(dv==0)	return;
 	long *kk=new long[n];
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 	for(long i=0;i<n;i++)
 	{
 		kk[i] = mgl_int((a[i]-v1)/dv-0.5);
@@ -1548,7 +1542,7 @@ void MGL_EXPORT mgl_data_coil(HMDT d, mreal v1, mreal v2, int sep)
 	}
 	if(sep)
 	{
-#pragma omp parallel for
+#pragma omp parallel for schedule(static)
 		for(long i=1;i<n;i++)	if(kk[i]!=kk[i-1])	a[i] = NAN;
 	}
 	delete []kk;
